# Copyright 2013 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from oslo_serialization import jsonutils

from tackerclient.common import exceptions as exception
from tackerclient.i18n import _

LOG = logging.getLogger(__name__)


class ActionDispatcher(object):
    """Maps method name to local methods through action name."""

    def dispatch(self, *args, **kwargs):
        """Find and call local method."""
        action = kwargs.pop('action', 'default')
        action_method = getattr(self, str(action), self.default)
        return action_method(*args, **kwargs)

    def default(self, data):
        raise NotImplementedError()


class DictSerializer(ActionDispatcher):
    """Default request body serialization."""

    def serialize(self, data, action='default'):
        return self.dispatch(data, action=action)

    def default(self, data):
        return ""


class JSONDictSerializer(DictSerializer):
    """Default JSON request body serialization."""

    def default(self, data):
        def sanitizer(obj):
            return str(obj)
        return jsonutils.dumps(data, default=sanitizer)


class TextDeserializer(ActionDispatcher):
    """Default request body deserialization."""

    def deserialize(self, datastring, action='default'):
        return self.dispatch(datastring, action=action)

    def default(self, datastring):
        return {}


class JSONDeserializer(TextDeserializer):

    def _from_json(self, datastring):
        try:
            return jsonutils.loads(datastring)
        except ValueError:
            msg = _("Cannot understand JSON")
            raise exception.MalformedResponseBody(reason=msg)

    def default(self, datastring):
        return {'body': self._from_json(datastring)}


# NOTE(maru): this class is duplicated from tacker.wsgi
class Serializer(object):
    """Serializes and deserializes dictionaries to certain MIME types."""

    def __init__(self, metadata=None):
        """Create a serializer based on the given WSGI environment.

        'metadata' is an optional dict mapping MIME types to information
        needed to serialize a dictionary to that type.

        """
        self.metadata = metadata or {}

    def _get_serialize_handler(self, content_type):
        handlers = {
            'application/json': JSONDictSerializer()
        }

        try:
            return handlers[content_type]
        except Exception:
            raise exception.InvalidContentType(content_type=content_type)

    def serialize(self, data, content_type):
        """Serialize a dictionary into the specified content type."""
        return self._get_serialize_handler(content_type).serialize(data)

    def deserialize(self, datastring, content_type):
        """Deserialize a string to a dictionary.

        The string must be in the format of a supported MIME type.
        """
        return self.get_deserialize_handler(content_type).deserialize(
            datastring)

    def get_deserialize_handler(self, content_type):
        handlers = {
            'application/json': JSONDeserializer()
        }

        try:
            return handlers[content_type]
        except Exception:
            raise exception.InvalidContentType(content_type=content_type)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""OpenStackClient plugin for nfv-orchestration service."""

import logging

from osc_lib import utils


LOG = logging.getLogger(__name__)

# Required by the OSC plugin interface
DEFAULT_TACKER_API_VERSION = '1'
API_NAME = 'tackerclient'
API_VERSION_OPTION = 'os_tacker_api_version'
API_VERSIONS = {
    '1': 'tackerclient.v1_0.client.Client',
    '2': 'tackerclient.v1_0.client.Client',
}


def make_client(instance):
    """Returns a client to the ClientManager."""

    api_version = instance._api_version[API_NAME]
    tacker_client = utils.get_client_class(
        API_NAME,
        api_version,
        API_VERSIONS)
    LOG.debug('Instantiating tacker client: %s', tacker_client)

    kwargs = {'service_type': 'nfv-orchestration',
              'region_name': instance._region_name,
              'endpoint_type': instance._interface,
              'interface': instance._interface,
              'session': instance.session,
              'api_version': api_version
              }

    client = tacker_client(**kwargs)

    return client


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--os-tacker-api-version',
        metavar='<tacker-api-version>',
        default=utils.env(
            'OS_TACKER_API_VERSION',
            default=DEFAULT_TACKER_API_VERSION))
    return parser



# Copyright 2016 NEC Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""This module should contain OSC plugin generic methods.

Methods in this module are candidates adopted to osc-lib.

Stuffs specific to tackerclient OSC plugin should not be added
to this module. They should go to tackerclient.osc.v1.utils.
"""

import json
import operator
import os

from cliff import columns as cliff_columns
from keystoneclient import exceptions as identity_exc
from keystoneclient.v3 import domains
from keystoneclient.v3 import projects
from osc_lib import utils
from oslo_serialization import jsonutils

from tackerclient.common import exceptions
from tackerclient.i18n import _


LIST_BOTH = 'both'
LIST_SHORT_ONLY = 'short_only'
LIST_LONG_ONLY = 'long_only'


def format_dict_with_indention(data):
    """Return a formatted string of key value pairs

    :param data: a dict
    :rtype: a string formatted to key='value'
    """

    if data is None:
        return None
    return jsonutils.dumps(data, indent=4)


def get_column_definitions(attr_map, long_listing):
    """Return table headers and column names for a listing table.

    :param attr_map: a list of table entry definitions.
      Each entry should be a tuple consisting of
      (API attribute name, header name, listing mode). For example:
      (('id', 'ID', LIST_BOTH),
       ('name', 'Name', LIST_BOTH),
       ('tenant_id', 'Project', LIST_LONG_ONLY))
      The third field of each tuple must be one of LIST_BOTH,
      LIST_LONG_ONLY (a corresponding column is shown only in a long mode), or
      LIST_SHORT_ONLY (a corresponding column is shown only in a short mode).
    :param long_listing: A boolean value which indicates a long listing
      or not. In most cases, parsed_args.long is passed to this argument.
    :return: A tuple of a list of table headers and a list of column names.
    """

    if long_listing:
        headers = [hdr for col, hdr, listing_mode in attr_map
                   if listing_mode in (LIST_BOTH, LIST_LONG_ONLY)]
        columns = [col for col, hdr, listing_mode in attr_map
                   if listing_mode in (LIST_BOTH, LIST_LONG_ONLY)]
    else:
        headers = [hdr for col, hdr, listing_mode in attr_map
                   if listing_mode in (LIST_BOTH, LIST_SHORT_ONLY)]
        columns = [col for col, hdr, listing_mode in attr_map
                   if listing_mode in (LIST_BOTH, LIST_SHORT_ONLY)]

    return headers, columns


def get_columns(item, attr_map=None):
    """Return pair of resource attributes and corresponding display names.

    Assume the following item and attr_map are passed.
    item: {'id': 'myid', 'name': 'myname',
           'foo': 'bar', 'tenant_id': 'mytenan'}
    attr_map:
      (('id', 'ID', LIST_BOTH),
       ('name', 'Name', LIST_BOTH),
       ('tenant_id', 'Project', LIST_LONG_ONLY))

    This method returns:

       (('id', 'name', 'tenant_id', 'foo'),  # attributes
        ('ID', 'Name', 'Project', 'foo')     # display names

    Both tuples of attributes and display names are sorted by display names
    in the alphabetical order.
    Attributes not found in a given attr_map are kept as-is.

    :param item: a dictionary which represents a resource.
      Keys of the dictionary are expected to be attributes of the resource.
      Values are not referred to by this method.
    :param attr_map: a list of mapping from attribute to display name.
      The same format is used as for get_column_definitions attr_map.
    :return: A pair of tuple of attributes and tuple of display names.
    """
    attr_map = attr_map or tuple([])
    _attr_map_dict = dict((col, hdr) for col, hdr, listing_mode in attr_map)

    columns = [(column, _attr_map_dict.get(column, column))
               for column in item.keys()]
    columns = sorted(columns, key=operator.itemgetter(1))
    return (tuple(col[0] for col in columns),
            tuple(col[1] for col in columns))


# TODO(amotoki): Use osc-lib version once osc-lib provides this.
def add_project_owner_option_to_parser(parser):
    """Register project and project domain options.

    :param parser: argparse.Argument parser object.

    """
    parser.add_argument(
        '--project',
        metavar='<project>',
        help=_("Owner's project (name or ID)")
    )
    # Borrowed from openstackclient.identity.common
    # as it is not exposed officially.
    parser.add_argument(
        '--project-domain',
        metavar='<project-domain>',
        help=_('Domain the project belongs to (name or ID). '
               'This can be used in case collisions between project names '
               'exist.'),
    )


# The following methods are borrowed from openstackclient.identity.common
# as it is not exposed officially.
# TODO(amotoki): Use osc-lib version once osc-lib provides this.


def find_domain(identity_client, name_or_id):
    return _find_identity_resource(identity_client.domains, name_or_id,
                                   domains.Domain)


def find_project(identity_client, name_or_id, domain_name_or_id=None):
    domain_id = _get_domain_id_if_requested(identity_client, domain_name_or_id)
    if not domain_id:
        return _find_identity_resource(identity_client.projects, name_or_id,
                                       projects.Project)
    else:
        return _find_identity_resource(identity_client.projects, name_or_id,
                                       projects.Project, domain_id=domain_id)


def _get_domain_id_if_requested(identity_client, domain_name_or_id):
    if not domain_name_or_id:
        return None
    domain = find_domain(identity_client, domain_name_or_id)
    return domain.id


def _find_identity_resource(identity_client_manager, name_or_id,
                            resource_type, **kwargs):
    """Find a specific identity resource.

    Using keystoneclient's manager, attempt to find a specific resource by its
    name or ID. If Forbidden to find the resource (a common case if the user
    does not have permission), then return the resource by creating a local
    instance of keystoneclient's Resource.

    The parameter identity_client_manager is a keystoneclient manager,
    for example: keystoneclient.v3.users or keystoneclient.v3.projects.

    The parameter resource_type is a keystoneclient resource, for example:
    keystoneclient.v3.users.User or keystoneclient.v3.projects.Project.

    :param identity_client_manager: the manager that contains the resource
    :type identity_client_manager: `keystoneclient.base.CrudManager`
    :param name_or_id: the resources's name or ID
    :type name_or_id: string
    :param resource_type: class that represents the resource type
    :type resource_type: `keystoneclient.base.Resource`

    :returns: the resource in question
    :rtype: `keystoneclient.base.Resource`

    """

    try:
        identity_resource = utils.find_resource(identity_client_manager,
                                                name_or_id, **kwargs)
        if identity_resource is not None:
            return identity_resource
    except identity_exc.Forbidden:
        pass

    return resource_type(None, {'id': name_or_id, 'name': name_or_id})


# The above are borrowed from openstackclient.identity.common.


class FormatComplexDataColumn(cliff_columns.FormattableColumn):

    def human_readable(self):
        return format_dict_with_indention(self._value)


def jsonfile2body(file_path):

    if file_path is None:
        msg = _("File %s does not exist")
        reason = msg % file_path
        raise exceptions.InvalidInput(reason=reason)

    if os.access(file_path, os.R_OK) is False:
        msg = _("User does not have read privileges to it")
        raise exceptions.InvalidInput(reason=msg)

    try:
        with open(file_path) as f:
            body = json.load(f)
    except (IOError, ValueError) as ex:
        msg = _("Failed to load parameter file. Error: %s")
        reason = msg % ex
        raise exceptions.InvalidInput(reason=reason)

    if not body:
        reason = _('The parameter file is empty')
        raise exceptions.EmptyInput(reason=reason)

    return body



# Copyright (C) 2022 Fujitsu
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import os

from oslo_utils.fixture import uuidsentinel
from unittest import mock

from tackerclient import client as root_client
from tackerclient.common import exceptions
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.osc.v2.vnffm import vnffm_alarm
from tackerclient.tests.unit.osc import base
from tackerclient.tests.unit.osc.v1.fixture_data import client
from tackerclient.tests.unit.osc.v2 import vnffm_alarm_fakes
from tackerclient.tests.unit.test_cli10 import MyResp
from tackerclient.v1_0 import client as proxy_client


class TestVnfFmAlarm(base.FixturedTestCase):
    client_fixture_class = client.ClientFixture

    def setUp(self):
        super(TestVnfFmAlarm, self).setUp()
        self.url = client.TACKER_URL
        self.header = {'content-type': 'application/json'}
        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = self.cs
        self.app.client_manager.tackerclient = self.client_manager


def _get_columns_vnffm_alarm(action=None):
    if action == 'update':
        columns = ['Ack State']
    else:
        columns = ['ID', 'Managed Object Id', 'Ack State',
                   'Perceived Severity', 'Event Type', 'Probable Cause']

    if action == 'show':
        columns.extend([
            'Vnfc Instance Ids', 'Root Cause Faulty Resource',
            'Alarm Raised Time', 'Alarm Changed Time',
            'Alarm Cleared Time', 'Alarm Acknowledged Time',
            'Event Time', 'Fault Type', 'Is Root Cause',
            'Correlated Alarm Ids', 'Fault Details', 'Links'
        ])

    return columns


class TestListVnfFmAlarm(TestVnfFmAlarm):

    def setUp(self):
        super(TestListVnfFmAlarm, self).setUp()
        self.list_vnf_fm_alarms = vnffm_alarm.ListVnfFmAlarm(
            self.app, self.app_args, cmd_name='vnffm alarm list')

    def test_take_action(self):
        vnffm_alarms_obj = vnffm_alarm_fakes.create_vnf_fm_alarms(
            count=3)
        parsed_args = self.check_parser(self.list_vnf_fm_alarms, [], [])
        self.requests_mock.register_uri(
            'GET', os.path.join(self.url, 'vnffm/v1/alarms'),
            json=vnffm_alarms_obj, headers=self.header)

        actual_columns, data = self.list_vnf_fm_alarms.take_action(parsed_args)

        _, columns = tacker_osc_utils.get_column_definitions(
            vnffm_alarm._ATTR_MAP, long_listing=True)

        expected_data = []
        for vnffm_alarm_obj_idx in vnffm_alarms_obj:
            expected_data.append(vnffm_alarm_fakes.get_vnffm_alarm_data(
                vnffm_alarm_obj_idx, columns=columns))

        self.assertCountEqual(_get_columns_vnffm_alarm(action='list'),
                              actual_columns)
        self.assertCountEqual(expected_data, list(data))

    def test_take_action_with_filter(self):
        vnffm_alarms_obj = vnffm_alarm_fakes.create_vnf_fm_alarms(
            count=3)
        parsed_args = self.check_parser(
            self.list_vnf_fm_alarms,
            ["--filter", '(eq,perceivedSeverity,WARNING)'],
            [('filter', '(eq,perceivedSeverity,WARNING)')])
        self.requests_mock.register_uri(
            'GET', os.path.join(
                self.url,
                'vnffm/v1/alarms?filter=(eq,perceivedSeverity,WARNING)'),
            json=vnffm_alarms_obj, headers=self.header)

        actual_columns, data = self.list_vnf_fm_alarms.take_action(parsed_args)

        _, columns = tacker_osc_utils.get_column_definitions(
            vnffm_alarm._ATTR_MAP, long_listing=True)

        expected_data = []
        for vnffm_alarm_obj_idx in vnffm_alarms_obj:
            expected_data.append(vnffm_alarm_fakes.get_vnffm_alarm_data(
                vnffm_alarm_obj_idx, columns=columns))

        self.assertCountEqual(_get_columns_vnffm_alarm(action='list'),
                              actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    def test_take_action_with_incorrect_filter(self):

        parsed_args = self.check_parser(
            self.list_vnf_fm_alarms,
            ["--filter", '(perceivedSeverity)'],
            [('filter', '(perceivedSeverity)')])

        url = os.path.join(
            self.url, 'vnffm/v1/alarms?filter=(perceivedSeverity)')

        self.requests_mock.register_uri(
            'GET', url, headers=self.header, status_code=400, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.list_vnf_fm_alarms.take_action,
                          parsed_args)

    def test_take_action_internal_server_error(self):

        parsed_args = self.check_parser(
            self.list_vnf_fm_alarms,
            ["--filter", '(eq,perceivedSeverity,WARNING)'],
            [('filter', '(eq,perceivedSeverity,WARNING)')])

        url = os.path.join(
            self.url, 'vnffm/v1/alarms?filter=(eq,perceivedSeverity,WARNING)')

        self.requests_mock.register_uri(
            'GET', url, headers=self.header, status_code=500, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.list_vnf_fm_alarms.take_action,
                          parsed_args)


class TestShowVnfFmAlarm(TestVnfFmAlarm):

    def setUp(self):
        super(TestShowVnfFmAlarm, self).setUp()
        self.show_vnf_fm_alarm = vnffm_alarm.ShowVnfFmAlarm(
            self.app, self.app_args, cmd_name='vnffm alarm show')

    def test_take_action(self):
        """Test of take_action()"""
        vnffm_alarm_obj = vnffm_alarm_fakes.vnf_fm_alarm_response()

        arglist = [vnffm_alarm_obj['id']]
        verifylist = [('vnf_fm_alarm_id', vnffm_alarm_obj['id'])]

        # command param
        parsed_args = self.check_parser(
            self.show_vnf_fm_alarm, arglist, verifylist)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', vnffm_alarm_obj['id'])

        self.requests_mock.register_uri(
            'GET', url, headers=self.header, json=vnffm_alarm_obj)

        columns, _ = (self.show_vnf_fm_alarm.take_action(parsed_args))

        self.assertCountEqual(_get_columns_vnffm_alarm(action='show'),
                              columns)

    def test_take_action_vnf_lcm_op_occ_id_not_found(self):
        """Test if vnf-lcm-op-occ-id does not find."""
        arglist = [uuidsentinel.vnf_fm_alarm_id]
        verifylist = [('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]

        # command param
        parsed_args = self.check_parser(
            self.show_vnf_fm_alarm, arglist, verifylist)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'GET', url, headers=self.header, status_code=404, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.show_vnf_fm_alarm.take_action,
                          parsed_args)

    def test_take_action_internal_server_error(self):
        """Test for internal server error."""
        arglist = [uuidsentinel.vnf_fm_alarm_id]
        verifylist = [('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]

        # command param
        parsed_args = self.check_parser(
            self.show_vnf_fm_alarm, arglist, verifylist)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'GET', url, headers=self.header, status_code=500, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.show_vnf_fm_alarm.take_action,
                          parsed_args)


@ddt.ddt
class TestUpdateVnfFmAlarm(TestVnfFmAlarm):

    def setUp(self):
        super(TestUpdateVnfFmAlarm, self).setUp()
        self.update_vnf_fm_alarm = vnffm_alarm.UpdateVnfFmAlarm(
            self.app, self.app_args, cmd_name='vnffm alarm update')

    @ddt.data('ACKNOWLEDGED', 'UNACKNOWLEDGED')
    def test_take_action(self, ack_state):
        """Test of take_action()"""

        vnffm_alarm_obj = vnffm_alarm_fakes.vnf_fm_alarm_response(
            None, 'update')

        arg_list = ['--ack-state', ack_state, uuidsentinel.vnf_fm_alarm_id]
        verify_list = [('ack_state', ack_state),
                       ('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]

        # command param
        parsed_args = self.check_parser(
            self.update_vnf_fm_alarm, arg_list, verify_list)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'PATCH', url, headers=self.header, json=vnffm_alarm_obj)

        actual_columns, data = (
            self.update_vnf_fm_alarm.take_action(parsed_args))

        expected_columns = _get_columns_vnffm_alarm(action='update')

        self.assertCountEqual(expected_columns, actual_columns)

        _, columns = vnffm_alarm._get_columns(
            vnffm_alarm_obj, action='update')

        expected_data = vnffm_alarm_fakes.get_vnffm_alarm_data(
            vnffm_alarm_obj, columns=columns)

        self.assertEqual(expected_data, data)

    @ddt.data('ACKNOWLEDGED')
    @mock.patch.object(proxy_client.ClientBase, 'deserialize')
    def test_take_action_check_content_type(self, ack_state, mock_des):
        """Check content type by test of take_action()"""

        vnffm_alarm_obj = vnffm_alarm_fakes.vnf_fm_alarm_response(
            None, 'update')

        arg_list = ['--ack-state', ack_state, uuidsentinel.vnf_fm_alarm_id]
        verify_list = [('ack_state', ack_state),
                       ('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]
        mock_des.return_value = {}

        # command param
        parsed_args = self.check_parser(
            self.update_vnf_fm_alarm, arg_list, verify_list)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'PATCH', url, headers=self.header, json=vnffm_alarm_obj)

        with mock.patch.object(root_client.HTTPClient,
                               'do_request') as mock_req:
            headers = {'Content-Type': 'application/json'}
            mock_req.return_value = (MyResp(200, headers=headers), None)
            self.update_vnf_fm_alarm.take_action(parsed_args)
            mock_req.assert_called_once_with(
                f'/vnffm/v1/alarms/{uuidsentinel.vnf_fm_alarm_id}',
                'PATCH', body=mock.ANY, headers=mock.ANY,
                content_type='application/merge-patch+json', accept='json')

    @ddt.data('ACKNOWLEDGED')
    def test_take_action_vnf_lcm_op_occ_id_not_found(self, ack_state):
        """Test if vnf-lcm-op-occ-id does not find"""

        arg_list = ['--ack-state', ack_state, uuidsentinel.vnf_fm_alarm_id]
        verify_list = [('ack_state', ack_state),
                       ('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]

        # command param
        parsed_args = self.check_parser(
            self.update_vnf_fm_alarm, arg_list, verify_list)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'PATCH', url, headers=self.header, status_code=404, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.update_vnf_fm_alarm.take_action,
                          parsed_args)

    @ddt.data('UNACKNOWLEDGED')
    def test_take_action_vnf_lcm_op_occ_state_is_conflict(self, ack_state):
        """Test if vnf-lcm-op-occ state is conflict"""

        arg_list = ['--ack-state', ack_state, uuidsentinel.vnf_fm_alarm_id]
        verify_list = [('ack_state', ack_state),
                       ('vnf_fm_alarm_id', uuidsentinel.vnf_fm_alarm_id)]

        # command param
        parsed_args = self.check_parser(
            self.update_vnf_fm_alarm, arg_list, verify_list)

        url = os.path.join(
            self.url, 'vnffm/v1/alarms', uuidsentinel.vnf_fm_alarm_id)

        self.requests_mock.register_uri(
            'PATCH', url, headers=self.header, status_code=409, json={})

        self.assertRaises(exceptions.TackerClientException,
                          self.update_vnf_fm_alarm.take_action,
                          parsed_args)



# Copyright (C) 2022 Fujitsu
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils

from tackerclient.osc import utils as tacker_osc_utils


def create_vnf_fm_subs(count=2):
    """Create multiple fake vnf packages.

    :param int count:
        The number of vnf_fm_subs to fake
    :return:
        A list of fake vnf fm subs dictionary
    """
    vnf_fm_subs = []
    for i in range(0, count):
        unique_id = uuidutils.generate_uuid()
        vnf_fm_subs.append(vnf_fm_sub_response(attrs={'id': unique_id}))
    return vnf_fm_subs


def vnf_fm_sub_response(attrs=None):
    """Create a fake vnf fm sub.

    :param Dictionary attrs:
        A dictionary with all attributes
    :return:
        A FakeVnfFmAlarm dict
    """

    attrs = attrs or {}
    # Set default attributes.
    fake_vnf_fm_sub = {
        "id": "78a39661-60a8-4824-b989-88c1b0c3534a",
        "filter": {
            "vnfInstanceSubscriptionFilter": {
                "vnfdIds": [
                    "dummy-vnfdId-1"
                ],
                "vnfProductsFromProviders": [
                    {
                        "vnfProvider": "dummy-vnfProvider-1",
                        "vnfProducts": [
                            {
                                "vnfProductName": "dummy-vnfProductName-1-1",
                                "versions": [
                                    {
                                        "vnfSoftwareVersion": 1.0,
                                        "vnfdVersions": [1.0, 2.0]
                                    }
                                ]
                            }
                        ]
                    }
                ],
                "vnfInstanceIds": [
                    "dummy-vnfInstanceId-1"
                ],
                "vnfInstanceNames": [
                    "dummy-vnfInstanceName-1"
                ]
            },
            "notificationTypes": [
                "AlarmNotification"
            ],
            "faultyResourceTypes": [
                "COMPUTE"
            ],
            "perceivedSeverities": [
                "WARNING"
            ],
            "eventTypes": [
                "EQUIPMENT_ALARM"
            ],
            "probableCauses": [
                "The server cannot be connected."
            ]
        },
        "callbackUri": "/nfvo/notify/alarm",
        "_links": {
            "self": {
                "href": "/vnffm/v1/subscriptions/"
                        "78a39661-60a8-4824-b989-88c1b0c3534a"
            }
        }
    }

    # Overwrite default attributes.
    fake_vnf_fm_sub.update(attrs)

    return fake_vnf_fm_sub


def get_vnffm_sub_data(vnf_fm_sub, columns=None):
    """Get the vnffm sub.

    :return:
        A tuple object sorted based on the name of the columns.
    """
    complex_attributes = ['filter', '_links']

    for attribute in complex_attributes:
        if vnf_fm_sub.get(attribute):
            vnf_fm_sub.update(
                {attribute: tacker_osc_utils.FormatComplexDataColumn(
                    vnf_fm_sub[attribute])})

    # return the list of data as per column order
    if columns:
        return tuple([vnf_fm_sub[key] for key in columns])

    return tuple([vnf_fm_sub[key] for key in sorted(
        vnf_fm_sub.keys())])



# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from functools import reduce
import logging
import sys

from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import utils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils

LOG = logging.getLogger(__name__)


formatters = {'softwareImages': tacker_osc_utils.FormatComplexDataColumn,
              'checksum': tacker_osc_utils.FormatComplexDataColumn,
              '_links': tacker_osc_utils.FormatComplexDataColumn,
              'userDefinedData': tacker_osc_utils.FormatComplexDataColumn,
              'additionalArtifacts': tacker_osc_utils.FormatComplexDataColumn}


_mixed_case_fields = ('usageState', 'onboardingState', 'operationalState',
                      'vnfProductName', 'softwareImages', 'userDefinedData',
                      'vnfdId', 'vnfdVersion', 'vnfSoftwareVersion',
                      'vnfProvider', 'additionalArtifacts')


def _get_columns(vnf_package_obj):
    column_map = {
        '_links': 'Links',
        'onboardingState': 'Onboarding State',
        'operationalState': 'Operational State',
        'usageState': 'Usage State',
        'userDefinedData': 'User Defined Data',
        'id': 'ID'
    }

    if vnf_package_obj['onboardingState'] == 'ONBOARDED':
        column_map.update({
            'softwareImages': 'Software Images',
            'vnfProvider': 'VNF Provider',
            'vnfSoftwareVersion': 'VNF Software Version',
            'vnfProductName': 'VNF Product Name',
            'vnfdId': 'VNFD ID',
            'vnfdVersion': 'VNFD Version',
            'checksum': 'Checksum',
            'additionalArtifacts': 'Additional Artifacts'
        })

    return sdk_utils.get_osc_show_columns_for_sdk_resource(vnf_package_obj,
                                                           column_map)


class CreateVnfPackage(command.ShowOne):
    _description = _("Create a new VNF Package")

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(CreateVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            '--user-data',
            metavar='<key=value>',
            action=parseractions.KeyValueAction,
            help=_('User defined data for the VNF package '
                   '(repeat option to set multiple user defined data)'),
        )
        return parser

    def args2body(self, parsed_args):
        body = {}
        if parsed_args.user_data:
            body["userDefinedData"] = parsed_args.user_data
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        vnf_package = client.create_vnf_package(self.args2body(parsed_args))
        display_columns, columns = _get_columns(vnf_package)
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnf_package),
            columns, formatters=formatters,
            mixed_case_fields=_mixed_case_fields)
        return (display_columns, data)


class ListVnfPackage(command.Lister):
    _description = _("List VNF Packages")

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(ListVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            "--filter",
            metavar="<filter>",
            help=_("Atrribute-based-filtering parameters"),
        )
        fields_exclusive_group = parser.add_mutually_exclusive_group(
            required=False)
        fields_exclusive_group.add_argument(
            "--all_fields",
            action="store_true",
            default=False,
            help=_("Include all complex attributes in the response"),
        )
        fields_exclusive_group.add_argument(
            "--fields",
            metavar="fields",
            help=_("Complex attributes to be included into the response"),
        )
        fields_exclusive_group.add_argument(
            "--exclude_fields",
            metavar="exclude-fields",
            help=_("Complex attributes to be excluded from the response"),
        )
        parser.add_argument(
            "--exclude_default",
            action="store_true",
            default=False,
            help=_("Indicates to exclude all complex attributes"
                   " from the response. This argument can be used alone or"
                   " with --fields and --filter. For all other combinations"
                   " tacker server will throw bad request error"),
        )
        return parser

    def case_modify(self, field):
        return reduce(
            lambda x, y: x + (' ' if y.isupper() else '') + y, field).title()

    def get_attributes(self, extra_fields=None, all_fields=False,
                       exclude_fields=None, exclude_default=False):
        fields = ['id', 'vnfProductName', 'onboardingState',
                  'usageState', 'operationalState', '_links']
        complex_fields = [
            'checksum',
            'softwareImages',
            'userDefinedData',
            'additionalArtifacts']
        simple_fields = ['vnfdVersion', 'vnfProvider', 'vnfSoftwareVersion',
                         'vnfdId']

        if extra_fields:
            fields.extend(extra_fields)

        if exclude_fields:
            fields.extend([field for field in complex_fields
                           if field not in exclude_fields])
        if all_fields:
            fields.extend(complex_fields)
            fields.extend(simple_fields)

        if exclude_default:
            fields.extend(simple_fields)

        attrs = []
        for field in fields:
            if field == '_links':
                attrs.extend([(field, 'Links', tacker_osc_utils.LIST_BOTH)])
            else:
                attrs.extend([(field, self.case_modify(field),
                               tacker_osc_utils.LIST_BOTH)])

        return tuple(attrs)

    def take_action(self, parsed_args):
        _params = {}
        extra_fields = []
        exclude_fields = []
        all_fields = False
        exclude_default = False
        if parsed_args.filter:
            _params['filter'] = parsed_args.filter
        if parsed_args.fields:
            _params['fields'] = parsed_args.fields
            fields = parsed_args.fields.split(',')
            for field in fields:
                extra_fields.append(field.split('/')[0])
        if parsed_args.exclude_fields:
            _params['exclude_fields'] = parsed_args.exclude_fields
            fields = parsed_args.exclude_fields.split(',')
            exclude_fields.extend(fields)
        if parsed_args.exclude_default:
            _params['exclude_default'] = None
            exclude_default = True
        if parsed_args.all_fields:
            _params['all_fields'] = None
            all_fields = True

        client = self.app.client_manager.tackerclient
        data = client.list_vnf_packages(**_params)
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.get_attributes(extra_fields, all_fields, exclude_fields,
                                exclude_default), long_listing=True)
        return (headers,
                (utils.get_dict_properties(
                    s, columns, formatters=formatters,
                    mixed_case_fields=_mixed_case_fields,
                ) for s in data['vnf_packages']))


class ShowVnfPackage(command.ShowOne):
    _description = _("Show VNF Package Details")

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(ShowVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            'vnf_package',
            metavar="<vnf-package>",
            help=_("VNF package ID")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        vnf_package = client.show_vnf_package(parsed_args.vnf_package)
        display_columns, columns = _get_columns(vnf_package)
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnf_package),
            columns, formatters=formatters,
            mixed_case_fields=_mixed_case_fields)
        return (display_columns, data)


class UploadVnfPackage(command.Command):
    _description = _("Upload VNF Package")

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(UploadVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            'vnf_package',
            metavar="<vnf-package>",
            help=_("VNF package ID")
        )
        file_source = parser.add_mutually_exclusive_group(required=True)
        file_source.add_argument(
            "--path",
            metavar="<file>",
            help=_("Upload VNF CSAR package from local file"),
        )
        file_source.add_argument(
            "--url",
            metavar="<Uri>",
            help=_("Uri of the VNF package content"),
        )
        parser.add_argument(
            "--user-name",
            metavar="<user-name>",
            help=_("User name for authentication"),
        )
        parser.add_argument(
            "--password",
            metavar="<password>",
            help=_("Password for authentication"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        attrs = {}
        if parsed_args.user_name:
            attrs['userName'] = parsed_args.user_name

        if parsed_args.password:
            attrs['password'] = parsed_args.password

        if parsed_args.url:
            attrs['url'] = parsed_args.url

        file_data = None
        try:
            if parsed_args.path:
                file_data = open(parsed_args.path, 'rb')
            result = client.upload_vnf_package(parsed_args.vnf_package,
                                               file_data, **attrs)
            if not result:
                print((_('Upload request for VNF package %(id)s has been'
                         ' accepted.') % {'id': parsed_args.vnf_package}))
        finally:
            if file_data:
                file_data.close()


class DeleteVnfPackage(command.Command):
    """Vnf package delete

    Delete class supports bulk deletion of vnf packages, and error
    handling.
    """

    _description = _("Delete VNF Package")

    resource = 'vnf-package'

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(DeleteVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            'vnf-package',
            metavar="<vnf-package>",
            nargs="+",
            help=_("Vnf package(s) ID to delete")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        resources = getattr(parsed_args, self.resource, [])
        for resource_id in resources:
            try:
                vnf_package = client.show_vnf_package(resource_id)
                client.delete_vnf_package(vnf_package['id'])
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': self.resource})
            err_msg = _("\n\nUnable to delete the below"
                        " 'vnf_package'(s):")
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': self.resource}))
        return


class DownloadVnfPackage(command.Command):
    _description = _("Download VNF package contents or VNFD of an on-boarded "
                     "VNF package.")

    def get_parser(self, prog_name):
        parser = super(DownloadVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            "vnf_package",
            metavar="<vnf-package>",
            help=_("VNF package ID")
        )
        parser.add_argument(
            "--file",
            metavar="<FILE>",
            help=_("Local file to save downloaded VNF Package or VNFD data. "
                   "If this is not specified and there is no redirection "
                   "then data will not be saved.")
        )
        parser.add_argument(
            "--vnfd",
            action="store_true",
            default=False,
            help=_("Download VNFD of an on-boarded vnf package."),
        )
        parser.add_argument(
            "--type",
            default="application/zip",
            metavar="<type>",
            choices=["text/plain", "application/zip", "both"],
            help=_("Provide text/plain when VNFD is implemented as a single "
                   "YAML file otherwise  use application/zip. If you are not "
                   "aware whether VNFD is a single or multiple yaml files, "
                   "then you can specify 'both' option value. "
                   "Provide this option only when --vnfd is set.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        if parsed_args.vnfd:
            if sys.stdout.isatty() and not (parsed_args.file and
                                            parsed_args.type != "text/plain"):
                msg = ("No redirection or local file specified for downloaded "
                       "VNFD data. Please specify a local file with --file to "
                       "save downloaded VNFD data or use redirection.")
                sdk_utils.exit(msg)

            body = client.download_vnfd_from_vnf_package(
                parsed_args.vnf_package, parsed_args.type)

            if not parsed_args.file:
                print(body)
                return
        else:
            body = client.download_vnf_package(parsed_args.vnf_package)

        sdk_utils.save_data(body, parsed_args.file)


class DownloadVnfPackageArtifact(command.Command):
    _description = _("Download VNF package artifact of an on-boarded "
                     "VNF package.")

    def get_parser(self, prog_name):
        parser = super(DownloadVnfPackageArtifact, self).get_parser(prog_name)
        parser.add_argument(
            "vnf_package",
            metavar="<vnf-package>",
            help=_("VNF package ID")
        )
        parser.add_argument(
            "artifact_path",
            metavar="<artifact-path>",
            help=_("The artifact file's path")
        )
        parser.add_argument(
            "--file",
            metavar="<FILE>",
            help=_("Local file to save downloaded VNF Package artifact "
                   "file data. If this is not specified and "
                   "there is no redirection then data will not be saved.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        if sys.stdout.isatty() and not (parsed_args.file):
            msg = (
                "No redirection or local file specified for downloaded "
                "vnf package artifact data. Please specify a "
                "local file with --file to "
                "save downloaded vnf package artifact data "
                "or use redirection.")
            sdk_utils.exit(msg)
        body = client.download_artifact_from_vnf_package(
            parsed_args.vnf_package, parsed_args.artifact_path)

        if not parsed_args.file:
            print(body)
            return
        else:
            sdk_utils.save_data(body, parsed_args.file)


class UpdateVnfPackage(command.ShowOne):
    _description = _("Update information about an individual VNF package")

    def get_parser(self, prog_name):
        LOG.debug('get_parser(%s)', prog_name)
        parser = super(UpdateVnfPackage, self).get_parser(prog_name)
        parser.add_argument(
            'vnf_package',
            metavar="<vnf-package>",
            help=_("VNF package ID")
        )
        parser.add_argument(
            '--operational-state',
            metavar="<operational-state>",
            choices=['ENABLED', 'DISABLED'],
            help=_("Change the operational state of VNF Package, Valid values"
                   " are 'ENABLED' or 'DISABLED'.")
        )
        parser.add_argument(
            '--user-data',
            metavar='<key=value>',
            action=parseractions.KeyValueAction,
            help=_('User defined data for the VNF package '
                   '(repeat option to set multiple user defined data)'),
        )
        return parser

    def get_columns(self, updated_values):
        column_map = {}
        if updated_values.get('userDefinedData'):
            column_map.update({'userDefinedData': 'User Defined Data'})

        if updated_values.get('operationalState'):
            column_map.update({'operationalState': 'Operational State'})

        return sdk_utils.get_osc_show_columns_for_sdk_resource(updated_values,
                                                               column_map)

    def args2body(self, parsed_args):
        body = {}
        if not parsed_args.user_data and not parsed_args.operational_state:
            msg = ('Provide at least one of the argument from "--user-data"'
                   ' or "--operational-state"')
            sdk_utils.exit(msg)
        if parsed_args.user_data:
            body["userDefinedData"] = parsed_args.user_data
        if parsed_args.operational_state:
            body["operationalState"] = parsed_args.operational_state
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        updated_values = client.update_vnf_package(
            parsed_args.vnf_package, self.args2body(parsed_args))
        display_columns, columns = self.get_columns(updated_values)
        data = utils.get_item_properties(
            sdk_utils.DictModel(updated_values),
            columns, formatters=formatters,
            mixed_case_fields=_mixed_case_fields)
        return (display_columns, data)



# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import filecmp
import os
import shutil
import sys
import tempfile
from unittest import mock

import ddt
import zipfile

from tackerclient import client as root_client
from tackerclient.common import exceptions
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.osc.v1.vnfpkgm import vnf_package
from tackerclient.tests.unit.osc import base
from tackerclient.tests.unit.osc.v1.fixture_data import client
from tackerclient.tests.unit.osc.v1 import vnf_package_fakes
from tackerclient.tests.unit.test_cli10 import MyResp
from tackerclient.v1_0 import client as proxy_client


def _create_zip():
    vnf_package_path = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                        'sample_vnf_package')
    tmpdir = tempfile.mkdtemp()
    tmparchive = os.path.join(tmpdir, 'sample_vnf_package')
    zip_file = shutil.make_archive(tmparchive, 'zip', vnf_package_path)
    return zip_file, tmpdir


def _get_columns_vnf_package(action='list', vnf_package_obj=None):
    columns = []
    if action == 'update':
        if vnf_package_obj.get('userDefinedData'):
            columns.extend(['User Defined Data'])
        if vnf_package_obj.get('operationalState'):
            columns.extend(['Operational State'])
        return columns

    columns.extend(['ID', 'Onboarding State', 'Operational State',
                    'Usage State', 'User Defined Data', 'Links'])

    if action in ['show', 'create']:
        if vnf_package_obj and vnf_package_obj[
            'onboardingState'] == 'ONBOARDED':
            columns.extend(['VNFD ID',
                            'VNF Provider',
                            'VNF Software Version',
                            'VNFD Version',
                            'Software Images',
                            'VNF Product Name',
                            'Checksum',
                            'Additional Artifacts'])

    return columns


class TestVnfPackage(base.FixturedTestCase):
    client_fixture_class = client.ClientFixture

    def setUp(self):
        super(TestVnfPackage, self).setUp()
        self.url = client.TACKER_URL
        self.header = {'content-type': 'application/json'}
        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = self.cs
        self.app.client_manager.tackerclient = self.client_manager


@ddt.ddt
class TestCreateVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestCreateVnfPackage, self).setUp()
        self.create_vnf_package = vnf_package.CreateVnfPackage(
            self.app, self.app_args, cmd_name='vnf package create')

    @ddt.data((["--user-data", 'Test_key=Test_value'],
               [('user_data', {'Test_key': 'Test_value'})]),
              ([], []))
    @ddt.unpack
    def test_take_action(self, arglist, verifylist):
        # command param
        parsed_args = self.check_parser(self.create_vnf_package, arglist,
                                        verifylist)

        if arglist:
            json = vnf_package_fakes.vnf_package_obj(
                attrs={'userDefinedData': {'Test_key': 'Test_value'}})
        else:
            json = vnf_package_fakes.vnf_package_obj()
        self.requests_mock.register_uri(
            'POST', self.url + '/vnfpkgm/v1/vnf_packages',
            json=json, headers=self.header)

        columns, data = (self.create_vnf_package.take_action(parsed_args))
        self.assertCountEqual(_get_columns_vnf_package(), columns)
        headers, attributes = vnf_package._get_columns(json)
        self.assertListItemsEqual(vnf_package_fakes.get_vnf_package_data(
            json, columns=attributes), data)


@ddt.ddt
class TestListVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestListVnfPackage, self).setUp()
        self.list_vnf_package = vnf_package.ListVnfPackage(
            self.app, self.app_args, cmd_name='vnf package list')
        self._vnf_packages = self._get_vnf_packages()

    def _get_vnf_packages(self, onboarded_vnf_package=False):
        return vnf_package_fakes.create_vnf_packages(
            count=3, onboarded_vnf_package=onboarded_vnf_package)

    def get_list_columns(self, all_fields=False, exclude_fields=None,
                         extra_fields=None, exclude_default=False):

        columns = ['Id', 'Vnf Product Name', 'Onboarding State', 'Usage State',
                   'Operational State', 'Links']
        complex_columns = [
            'Checksum',
            'Software Images',
            'User Defined Data',
            'Additional Artifacts']
        simple_columns = ['Vnfd Version', 'Vnf Provider', 'Vnfd Id',
                          'Vnf Software Version']

        if extra_fields:
            columns.extend(extra_fields)

        if exclude_fields:
            columns.extend([field for field in complex_columns
                           if field not in exclude_fields])
        if all_fields:
            columns.extend(complex_columns)
            columns.extend(simple_columns)

        if exclude_default:
            columns.extend(simple_columns)

        return columns

    def _get_mock_response_for_list_vnf_packages(
            self, filter_attribute, json=None):
        self.requests_mock.register_uri(
            'GET', self.url + '/vnfpkgm/v1/vnf_packages?' + filter_attribute,
            json=json if json else self._get_vnf_packages(),
            headers=self.header)

    def test_take_action_default_fields(self):
        parsed_args = self.check_parser(self.list_vnf_package, [], [])
        self.requests_mock.register_uri(
            'GET', self.url + '/vnfpkgm/v1/vnf_packages',
            json=self._vnf_packages, headers=self.header)
        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(), long_listing=True)

        for vnf_package_obj in self._vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True))
        self.assertCountEqual(self.get_list_columns(), actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    @ddt.data('all_fields', 'exclude_default')
    def test_take_action(self, arg):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--" + arg, "--filter", '(eq,onboardingState,ONBOARDED)'],
            [(arg, True), ('filter', '(eq,onboardingState,ONBOARDED)')])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        self._get_mock_response_for_list_vnf_packages(
            'filter=(eq,onboardingState,ONBOARDED)&' + arg, json=vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        kwargs = {arg: True}
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(**kwargs), long_listing=True)

        for vnf_package_obj in vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True, **kwargs))

        self.assertCountEqual(self.get_list_columns(**kwargs), actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    def test_take_action_with_pagination(self):
        next_links_num = 3
        parsed_args = self.check_parser(self.list_vnf_package, [], [])

        path = os.path.join(self.url, '/vnfpkgm/v1/vnf_packages?')

        links = [0] * next_links_num
        link_headers = [0] * next_links_num

        for i in range(next_links_num):
            links[i] = (
                '{base_url}?nextpage_opaque_marker={vnf_package_id}'.format(
                    base_url=path,
                    vnf_package_id=self._vnf_packages
                    ['vnf_packages'][i]['id']))
            link_headers[i] = copy.deepcopy(self.header)
            link_headers[i]['Link'] = '<{link_url}>; rel="next"'.format(
                link_url=links[i])

        self.requests_mock.register_uri(
            'GET', path, json=[self._vnf_packages['vnf_packages'][0]],
            headers=link_headers[0])
        self.requests_mock.register_uri(
            'GET', links[0], json=[self._vnf_packages['vnf_packages'][1]],
            headers=link_headers[1])
        self.requests_mock.register_uri(
            'GET', links[1], json=[self._vnf_packages['vnf_packages'][2]],
            headers=link_headers[2])
        self.requests_mock.register_uri(
            'GET', links[2], json=[], headers=self.header)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)

        kwargs = {}
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(**kwargs), long_listing=True)

        expected_data = []
        for vnf_package_obj in self._vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True, **kwargs))

        self.assertCountEqual(self.get_list_columns(**kwargs), actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    def test_take_action_with_exclude_fields(self):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--exclude_fields", 'softwareImages,checksum,'
                                 'userDefinedData,additionalArtifacts',
             "--filter", '(eq,onboardingState,ONBOARDED)'],
            [('exclude_fields', 'softwareImages,checksum,'
                                'userDefinedData,additionalArtifacts'),
             ('filter', '(eq,onboardingState,ONBOARDED)')])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        updated_vnf_packages = {'vnf_packages': []}
        for vnf_pkg in vnf_packages['vnf_packages']:
            vnf_pkg.pop('softwareImages')
            vnf_pkg.pop('checksum')
            vnf_pkg.pop('userDefinedData')
            vnf_pkg.pop('additionalArtifacts')
            updated_vnf_packages['vnf_packages'].append(vnf_pkg)
        self._get_mock_response_for_list_vnf_packages(
            'filter=(eq,onboardingState,ONBOARDED)&'
            'exclude_fields=softwareImages,checksum,'
            'userDefinedData,additionalArtifacts',
            json=updated_vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(
                exclude_fields=['softwareImages', 'checksum',
                                'userDefinedData', 'additionalArtifacts']),
            long_listing=True)

        for vnf_package_obj in updated_vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True))
        expected_columns = self.get_list_columns(
            exclude_fields=['Software Images', 'Checksum',
                            'User Defined Data', 'Additional Artifacts'])
        self.assertCountEqual(expected_columns, actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    @ddt.data((['--all_fields', '--fields', 'softwareImages'],
               [('all_fields', True), ('fields', 'softwareImages')]),
              (['--all_fields', '--exclude_fields', 'checksum'],
               [('all_fields', True), ('exclude_fields', 'checksum')]),
              (['--fields', 'softwareImages', '--exclude_fields', 'checksum'],
               [('fields', 'softwareImages'), ('exclude_fields', 'checksum')]))
    @ddt.unpack
    def test_take_action_with_invalid_combination(self, arglist, verifylist):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.list_vnf_package, arglist, verifylist)

    def test_take_action_with_valid_combination(self):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--fields", 'softwareImages,checksum', "--exclude_default"],
            [('fields', 'softwareImages,checksum'), ('exclude_default', True)])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        updated_vnf_packages = {'vnf_packages': []}
        for vnf_pkg in vnf_packages['vnf_packages']:
            vnf_pkg.pop('userDefinedData')
            updated_vnf_packages['vnf_packages'].append(vnf_pkg)

        self._get_mock_response_for_list_vnf_packages(
            'exclude_default&fields=softwareImages,checksum',
            json=updated_vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(
                extra_fields=['softwareImages', 'checksum'],
                exclude_default=True),
            long_listing=True)

        for vnf_package_obj in updated_vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True,
                exclude_default=True))

        self.assertCountEqual(self.get_list_columns(
            extra_fields=['Software Images', 'Checksum'],
            exclude_default=True),
            actual_columns)
        self.assertListItemsEqual(expected_data, list(data))


@ddt.ddt
class TestShowVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestShowVnfPackage, self).setUp()
        self.show_vnf_package = vnf_package.ShowVnfPackage(
            self.app, self.app_args, cmd_name='vnf package show')

    @ddt.data(True, False)
    def test_take_action(self, onboarded):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=onboarded)
        arglist = [vnf_package_obj['id']]
        verifylist = [('vnf_package', vnf_package_obj['id'])]
        parsed_args = self.check_parser(self.show_vnf_package, arglist,
                                        verifylist)
        url = self.url + '/vnfpkgm/v1/vnf_packages/' + vnf_package_obj['id']
        self.requests_mock.register_uri('GET', url, json=vnf_package_obj,
                                        headers=self.header)
        columns, data = (self.show_vnf_package.take_action(parsed_args))
        self.assertCountEqual(_get_columns_vnf_package(
            vnf_package_obj=vnf_package_obj, action='show'), columns)

        headers, attributes = vnf_package._get_columns(vnf_package_obj)
        self.assertListItemsEqual(
            vnf_package_fakes.get_vnf_package_data(vnf_package_obj,
                                                   columns=attributes), data)

    def test_show_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.show_vnf_package, [], [])


class TestDeleteVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestDeleteVnfPackage, self).setUp()
        self.delete_vnf_package = vnf_package.DeleteVnfPackage(
            self.app, self.app_args, cmd_name='vnf package delete')

        # The Vnf Package to delete
        self._vnf_package = vnf_package_fakes.create_vnf_packages(count=3)

    def _mock_request_url_for_delete(self, vnf_pkg_index):
        url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
               self._vnf_package['vnf_packages'][vnf_pkg_index]['id'])

        json = self._vnf_package['vnf_packages'][vnf_pkg_index]

        self.requests_mock.register_uri('GET', url, json=json,
                                        headers=self.header)
        self.requests_mock.register_uri('DELETE', url,
                                        headers=self.header, json={})

    def test_delete_one_vnf_package(self):
        arglist = [self._vnf_package['vnf_packages'][0]['id']]
        verifylist = [('vnf-package', [self._vnf_package['vnf_packages']
                                       [0]['id']])]

        parsed_args = self.check_parser(self.delete_vnf_package, arglist,
                                        verifylist)

        self._mock_request_url_for_delete(0)
        result = self.delete_vnf_package.take_action(parsed_args)
        self.assertIsNone(result)

    def test_delete_multiple_vnf_package(self):
        arglist = []
        for vnf_pkg in self._vnf_package['vnf_packages']:
            arglist.append(vnf_pkg['id'])
        verifylist = [('vnf-package', arglist)]
        parsed_args = self.check_parser(self.delete_vnf_package, arglist,
                                        verifylist)
        for i in range(0, 3):
            self._mock_request_url_for_delete(i)

        result = self.delete_vnf_package.take_action(parsed_args)
        self.assertIsNone(result)

    def test_delete_multiple_vnf_package_exception(self):
        arglist = [
            self._vnf_package['vnf_packages'][0]['id'],
            'xxxx-yyyy-zzzz',
            self._vnf_package['vnf_packages'][1]['id'],
        ]
        verifylist = [
            ('vnf-package', arglist),
        ]
        parsed_args = self.check_parser(self.delete_vnf_package,
                                        arglist, verifylist)

        self._mock_request_url_for_delete(0)

        url = (self.url + '/vnfpkgm/v1/vnf_packages/' + 'xxxx-yyyy-zzzz')
        body = {"error": exceptions.NotFound('404')}
        self.requests_mock.register_uri('GET', url, body=body,
                                        status_code=404, headers=self.header)
        self._mock_request_url_for_delete(1)
        self.assertRaises(exceptions.CommandError,
                          self.delete_vnf_package.take_action,
                          parsed_args)


@ddt.ddt
class TestUploadVnfPackage(TestVnfPackage):

    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestUploadVnfPackage, self).setUp()
        self.upload_vnf_package = vnf_package.UploadVnfPackage(
            self.app, self.app_args, cmd_name='vnf package upload')

    def test_upload_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.upload_vnf_package, [], [])

    def _mock_request_url_for_upload(self, method, status_code=202, body={}):
        if method == 'PUT':
            self.header = {'content-type': 'application/zip'}
            url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
                   self._vnf_package['id'] + '/package_content')
        else:
            url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
                   self._vnf_package['id'] + '/package_content/'
                                             'upload_from_uri')

        self.requests_mock.register_uri(method, url, json=body,
                                        headers=self.header,
                                        status_code=status_code)

    def _get_arglist_and_verifylist(self, method, path):
        if method == 'path':
            arglist = [
                self._vnf_package['id'],
                "--path", path
            ]
            verifylist = [
                ('path', path),
                ('vnf_package', self._vnf_package['id'])
            ]
        else:
            arglist = [
                self._vnf_package['id'],
                "--url", "http://uri:8000/test.zip",
                "--user-name", "Test_username",
                "--password", "Test_passoword",
            ]
            verifylist = [
                ('url', "http://uri:8000/test.zip"),
                ('user_name', 'Test_username'),
                ('password', 'Test_passoword'),
                ('vnf_package', self._vnf_package['id'])
            ]
        return arglist, verifylist

    @ddt.data('path', 'url')
    def test_upload_vnf_package(self, method):
        path = None
        if method == 'path':
            zip_file, temp_dir = _create_zip()
            path = zip_file

        arglist, verifylist = self._get_arglist_and_verifylist(method,
                                                               path)
        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)
        with mock.patch.object(proxy_client.ClientBase,
                               '_handle_fault_response') as m:
            if method == 'url':
                self._mock_request_url_for_upload('POST')
                self.upload_vnf_package.take_action(parsed_args)
            else:
                self._mock_request_url_for_upload('PUT')
                self.upload_vnf_package.take_action(parsed_args)
                # Delete temporary folder
                shutil.rmtree(temp_dir)
        # check no fault response is received
        self.assertNotCalled(m)

    @ddt.data('path')
    @mock.patch.object(proxy_client.ClientBase, 'deserialize')
    def test_upload_vnf_package_check_content_type(self, method, mock_des):
        path = None
        if method == 'path':
            zip_file, temp_dir = _create_zip()
            path = zip_file

        arglist, verifylist = self._get_arglist_and_verifylist(method, path)
        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)
        mock_des.return_value = {}
        with mock.patch.object(root_client.HTTPClient,
                               'do_request') as mock_req:
            headers = {'Content-Type': 'application/json'}
            mock_req.return_value = (MyResp(202, headers=headers), None)
            self._mock_request_url_for_upload('PUT')
            self.upload_vnf_package.take_action(parsed_args)
            # Delete temporary folder
            shutil.rmtree(temp_dir)
            mock_req.assert_called_once_with(
                f'/vnfpkgm/v1/vnf_packages/{self._vnf_package["id"]}'
                '/package_content', 'PUT',
                body=mock.ANY, headers=mock.ANY,
                content_type='application/zip', accept='json')

    def test_upload_vnf_package_with_conflict_error(self):
        # Scenario in which vnf package is already in on-boarded state
        zip_file, temp_dir = _create_zip()
        arglist, verifylist = self._get_arglist_and_verifylist('path',
                                                               zip_file)

        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)

        body = {"conflictingRequest": {
            "message": "VNF Package " + self._vnf_package['id'] +
                       " onboarding state is not CREATED", "code": 409}}
        self._mock_request_url_for_upload('PUT', status_code=409, body=body)

        self.assertRaises(exceptions.TackerClientException,
                          self.upload_vnf_package.take_action, parsed_args)
        # Delete temporary folder
        shutil.rmtree(temp_dir)

    def test_upload_vnf_package_failed_with_404_not_found(self):
        # Scenario in which vnf package is not found
        zip_file, temp_dir = _create_zip()
        arglist = [
            'dumy-id',
            "--path", zip_file
        ]
        verifylist = [
            ('path', zip_file),
            ('vnf_package', 'dumy-id')
        ]

        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)

        error_message = "Can not find requested vnf package: dummy-id"
        body = {"itemNotFound": {"message": error_message, "code": 404}}
        url = self.url + '/vnfpkgm/v1/vnf_packages/dumy-id/package_content'

        self.requests_mock.register_uri(
            'PUT', url, json=body,
            status_code=404)

        exception = self.assertRaises(
            exceptions.TackerClientException,
            self.upload_vnf_package.take_action, parsed_args)

        self.assertEqual(error_message, exception.message)
        # Delete temporary folder
        shutil.rmtree(temp_dir)


@ddt.ddt
class TestUpdateVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestUpdateVnfPackage, self).setUp()
        self.update_vnf_package = vnf_package.UpdateVnfPackage(
            self.app, self.app_args, cmd_name='vnf package update')

    @ddt.data((["--user-data", 'Test_key=Test_value',
                "--operational-state", 'DISABLED'],
               [('user_data', {'Test_key': 'Test_value'}),
                ('operational_state', 'DISABLED')]),
              (["--user-data", 'Test_key=Test_value'],
               [('user_data', {'Test_key': 'Test_value'})]),
              (["--operational-state", 'DISABLED'],
               [('operational_state', 'DISABLED')]))
    @ddt.unpack
    def test_take_action(self, arglist, verifylist):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=True)
        arglist.append(vnf_package_obj['id'])
        verifylist.append(('vnf_package', vnf_package_obj['id']))

        parsed_args = self.check_parser(self.update_vnf_package, arglist,
                                        verifylist)
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           vnf_package_obj['id'])
        fake_response = vnf_package_fakes.get_fake_update_vnf_package_obj(
            arglist)
        self.requests_mock.register_uri('PATCH', url,
                                        json=fake_response,
                                        headers=self.header)

        columns, data = self.update_vnf_package.take_action(parsed_args)
        self.assertCountEqual(_get_columns_vnf_package(
            vnf_package_obj=fake_response, action='update'), columns)
        self.assertListItemsEqual(
            vnf_package_fakes.get_vnf_package_data(fake_response), data)

    @ddt.data((["--user-data", 'Test_key=Test_value'],
               [('user_data', {'Test_key': 'Test_value'})]))
    @ddt.unpack
    @mock.patch.object(proxy_client.ClientBase, 'deserialize')
    def test_take_action_check_content_type(
            self, arglist, verifylist, mock_des):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=True)
        arglist.append(vnf_package_obj['id'])
        verifylist.append(('vnf_package', vnf_package_obj['id']))
        mock_des.return_value = {}

        parsed_args = self.check_parser(self.update_vnf_package, arglist,
                                        verifylist)
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           vnf_package_obj['id'])
        fake_response = vnf_package_fakes.get_fake_update_vnf_package_obj(
            arglist)
        self.requests_mock.register_uri('PATCH', url, json=fake_response,
                                        headers=self.header)

        with mock.patch.object(root_client.HTTPClient,
                               'do_request') as mock_req:
            headers = {'Content-Type': 'application/json'}
            mock_req.return_value = (MyResp(200, headers=headers), None)
            self.update_vnf_package.take_action(parsed_args)
            mock_req.assert_called_once_with(
                f'/vnfpkgm/v1/vnf_packages/{vnf_package_obj["id"]}', 'PATCH',
                body=mock.ANY, headers=mock.ANY,
                content_type='application/merge-patch+json', accept='json')

    def test_update_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.update_vnf_package, [], [])

    def test_update_without_user_data_and_operational_state(self):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=True)
        arglist = [vnf_package_obj['id']]

        verifylist = [('vnf_package', vnf_package_obj['id'])]

        parsed_args = self.check_parser(self.update_vnf_package, arglist,
                                        verifylist)
        self.assertRaises(SystemExit, self.update_vnf_package.take_action,
                          parsed_args)


@ddt.ddt
class TestDownloadVnfPackage(TestVnfPackage):

    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestDownloadVnfPackage, self).setUp()
        self.download_vnf_package = vnf_package.DownloadVnfPackage(
            self.app, self.app_args, cmd_name='vnf package download')

    def test_download_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.download_vnf_package, [], [])

    def _mock_request_url_for_download_vnfd(self, content_type, vnfd_data):
        self.header = {'content-type': content_type}
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'vnfd')

        if content_type == 'text/plain':
            self.requests_mock.register_uri('GET', url,
                                            headers=self.header,
                                            text=vnfd_data)
        else:
            self.requests_mock.register_uri('GET', url,
                                            headers=self.header,
                                            content=vnfd_data)

    def _get_arglist_and_verifylist(self, accept_type, file_name):
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', accept_type,
            '--file', file_name
        ]
        verifylist = [
            ('type', accept_type),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
            ('file', file_name)
        ]
        return arglist, verifylist

    def test_download_vnfd_from_vnf_package_for_type_text_plain(self):
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package/Definitions/'
                     'etsi_nfv_sol001_common_types.yaml')

        local_file = tempfile.NamedTemporaryFile(suffix='vnfd_data.yaml')
        vnfd_data = open(test_file, 'r').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            'text/plain', local_file.name)
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        self._mock_request_url_for_download_vnfd('text/plain', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")

    @ddt.data('application/zip', 'both')
    def test_download_vnfd_from_vnf_package(self, accept_type):
        test_file, temp_dir = _create_zip()
        # File in which VNFD data will be stored.
        # For testing purpose we are creating temporary zip file.
        local_file = tempfile.NamedTemporaryFile(suffix='vnfd_data.zip')
        vnfd_data = open(test_file, 'rb').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            accept_type, local_file.name)
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        # When --type argument is set to 'both', then 'Accept' header in
        # request is set to 'text/plain,application/zip' now it is up to the
        # NFVO to choose the format to return for a single-file VNFD and for
        # a multi-file VNFD, a ZIP file shall be returned. Here we have taken
        # the example of multi-file vnfd hence its retuning zip file and
        # setting the 'Content-Type' as 'application/zip' in response header.
        self._mock_request_url_for_download_vnfd('application/zip', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")
        self.assertTrue(self._check_valid_zip_file(local_file.name))
        shutil.rmtree(temp_dir)

    def _check_valid_zip_file(self, zip_file):
        with zipfile.ZipFile(zip_file) as zf:
            ret = zf.testzip()
        return False if ret else True

    @mock.patch('builtins.print')
    def test_download_vnfd_from_vnf_package_without_file_arg(self, mock_print):
        # --file argument is optional when --type is set to 'text/plain'.
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', 'text/plain',
        ]
        verifylist = [
            ('type', 'text/plain'),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
        ]
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package/Definitions/'
                     'etsi_nfv_sol001_common_types.yaml')

        vnfd_data = open(test_file, 'r').read()
        self._mock_request_url_for_download_vnfd('text/plain', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        mock_print.assert_called_once_with(vnfd_data)

    @ddt.data('application/zip', 'both')
    def test_download_vnfd_from_vnf_package_failed_with_no_file_arg(
            self, accept_type):
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', accept_type,
        ]
        verifylist = [
            ('type', accept_type),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
        ]
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        with mock.patch.object(sys.stdout, "isatty") as mock_isatty:
            mock_isatty.return_value = True
            self.assertRaises(SystemExit,
                              self.download_vnf_package.take_action,
                              parsed_args)

    def test_download_vnf_package(self):
        file_name = 'vnf_package_data.zip'
        test_file, temp_dir = _create_zip()

        # file in which VNF Package data will be stored.
        # for testing purpose we are creating temporary zip file.
        local_file = tempfile.NamedTemporaryFile(suffix=file_name)
        vnf_package_data = open(test_file, 'rb').read()

        arglist = [
            self._vnf_package['id'],
            '--file', local_file.name
        ]
        verifylist = [
            ('vnf_package', self._vnf_package['id']),
            ('file', local_file.name)
        ]

        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)

        url = os.path.join(self.url, '/vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'package_content')

        self.requests_mock.register_uri(
            'GET', url, headers={'content-type': 'application/zip'},
            content=vnf_package_data)

        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")
        self.assertTrue(self._check_valid_zip_file(local_file.name))
        shutil.rmtree(temp_dir)


@ddt.ddt
class TestDownloadVnfPackageArtifact(TestVnfPackage):
    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestDownloadVnfPackageArtifact, self).setUp()
        self.download_vnf_package_artifacts = vnf_package.\
            DownloadVnfPackageArtifact(
                self.app, self.app_args,
                cmd_name='vnf package artifact download')

    def test_download_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.download_vnf_package_artifacts, [], [])

    def _mock_request_url_for_download_artifacts(
        self, artifact_path, artifact_data):
        self.header = {'content-type': 'text/plain'}
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'artifacts', artifact_path)
        self.requests_mock.register_uri('GET', url,
                                        headers=self.header,
                                        text=artifact_data)

    def _get_arglist_and_verifylist(self, localfile):
        arglist = [
            self._vnf_package['id'],
            localfile.name[1:],
            '--file', localfile.name
        ]
        verifylist = [
            ('vnf_package', self._vnf_package['id']),
            ('artifact_path', localfile.name[1:]),
            ('file', localfile.name)
        ]
        return arglist, verifylist

    def test_download_artifacts_from_vnf_package(self):
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package_artifacts/Scripts/'
                     'install.sh')

        local_file = tempfile.NamedTemporaryFile(suffix='install.sh')
        artifact_data = open(test_file, 'r').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            local_file)
        parsed_args = self.check_parser(
            self.download_vnf_package_artifacts, arglist, verifylist)
        self._mock_request_url_for_download_artifacts(
            local_file.name[1:], artifact_data)
        self.download_vnf_package_artifacts.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")



# Copyright 2013 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

import testtools

from tackerclient.tacker import v1_0 as tackerV10


class TestCommandMeta(testtools.TestCase):
    def test_tacker_command_meta_defines_log(self):
        class FakeCommand(tackerV10.TackerCommand):
            pass

        self.assertTrue(hasattr(FakeCommand, 'log'))
        self.assertIsInstance(FakeCommand.log, logging.getLoggerClass())
        self.assertEqual(FakeCommand.log.name, __name__ + ".FakeCommand")

    def test_tacker_command_log_defined_explicitly(self):
        class FakeCommand(tackerV10.TackerCommand):
            log = None

        self.assertTrue(hasattr(FakeCommand, 'log'))
        self.assertIsNone(FakeCommand.log)



# Copyright (C) 2023 Fujitsu
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils

from tackerclient.osc import utils as tacker_osc_utils


def create_vnf_pm_thresholds(count=2):
    """Create multiple fake vnf pm thresholds.

    :param int count:
        The number of vnf_pm_thresholds to fake
    :return:
        A list of fake vnf pm thresholds dictionary
    """
    vnf_pm_thresholds = []
    for _ in range(0, count):
        unique_id = uuidutils.generate_uuid()
        vnf_pm_thresholds.append(vnf_pm_threshold_response(
            attrs={'id': unique_id}))
    return vnf_pm_thresholds


def vnf_pm_threshold_response(attrs=None, action=None):
    """Create a fake vnf pm threshold.

    :param Dictionary attrs:
        A dictionary with all attributes
    :param String action:
        The operation performed on threshold
    :return:
        A pm threshold dict
    """
    if action == 'update':
        fake_vnf_pm_threshold = {
            "callbackUri": "/nfvo/notify/threshold",
        }
        return fake_vnf_pm_threshold

    attrs = attrs or {}
    # Set default attributes.
    fake_vnf_pm_threshold = {
        "id": "2bb72d78-b1d9-48fe-8c64-332654ffeb5d",
        "objectType": "Vnfc",
        "objectInstanceId": "object-instance-id-1",
        "subObjectInstanceIds": [
            "sub-object-instance-id-2"
        ],
        "criteria": {
            "performanceMetric": "VCpuUsageMeanVnf.object-instance-id-1",
            "thresholdType": "SIMPLE",
            "simpleThresholdDetails": {
                "thresholdValue": 500.5,
                "hysteresis": 10.5
            }
        },
        "callbackUri": "/nfvo/notify/threshold",
        "_links": {
            "self": {
                "href": "/vnfpm/v2/thresholds/"
                        "78a39661-60a8-4824-b989-88c1b0c3534a"
            },
            "object": {
                "href": "/vnflcm/v1/vnf_instances/"
                        "0e5f3086-4e79-47ed-a694-54c29155fa26"
            }
        }
    }

    # Overwrite default attributes.
    fake_vnf_pm_threshold.update(attrs)

    return fake_vnf_pm_threshold


def get_vnfpm_threshold_data(vnf_pm_threshold, columns=None):
    """Get the vnfpm threshold.

    :param Dictionary vnf_pm_threshold:
        A dictionary with vnf_pm_threshold
    :param List columns:
        A list of column names
    :return:
        A tuple object sorted based on the name of the columns.
    """
    complex_attributes = ['subObjectInstanceIds',
                          'criteria', '_links', 'authentication']

    for attribute in complex_attributes:
        if vnf_pm_threshold.get(attribute):
            vnf_pm_threshold.update(
                {attribute: tacker_osc_utils.FormatComplexDataColumn(
                    vnf_pm_threshold[attribute])})

    # return the list of data as per column order
    if columns is None:
        columns = sorted(vnf_pm_threshold.keys())
    return tuple([vnf_pm_threshold[key] for key in columns])



# Copyright 2015-2016 Brocade Communications Systems Inc
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from tackerclient.common import exceptions
from tackerclient.common import utils
from tackerclient.tacker.v1_0.nfvo import vim
from tackerclient.tests.unit import test_cli10

API_VERSION = "1.0"
FORMAT = 'json'
TOKEN = 'testtoken'
ENDURL = 'localurl'


class CLITestV10VIMJSON(test_cli10.CLITestV10Base):
    _RESOURCE = 'vim'
    _RESOURCES = 'vims'

    def setUp(self):
        plurals = {'vims': 'vim'}
        super(CLITestV10VIMJSON, self).setUp(plurals=plurals)
        self.vim_project = {
            'name': 'abc',
            'project_domain_name': 'prj_domain_name'}
        self.auth_cred = {'username': 'xyz', 'password': '12345',
                          'user_domain_name': 'user_domain_name',
                          'cert_verify': 'True'}
        self.auth_url = 'http://1.2.3.4:5000'
        self.type = 'openstack'

    def test_register_vim_all_params(self):
        cmd = vim.CreateVIM(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        my_id = 'my-id'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        position_names = ['auth_cred', 'vim_project', 'auth_url', 'type']
        position_values = [self.auth_cred, self.vim_project,
                           self.auth_url, self.type]
        extra_body = {'type': 'openstack', 'name': name,
                      'description': description, 'is_default': False}
        self._test_create_resource(self._RESOURCE, cmd, None, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    def test_register_vim_with_false_cert_verify(self):
        cmd = vim.CreateVIM(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        my_id = 'my-id'
        # change cert_verify to False
        self.auth_cred = {'username': 'xyz', 'password': '12345',
                          'user_domain_name': 'user_domain_name',
                          'cert_verify': 'False'}
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config_with_false_cert_verify.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        position_names = ['auth_cred', 'vim_project', 'auth_url', 'type']
        position_values = [self.auth_cred, self.vim_project,
                           self.auth_url, self.type]
        extra_body = {'type': 'openstack', 'name': name,
                      'description': description, 'is_default': False}
        self._test_create_resource(self._RESOURCE, cmd, None, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    def test_register_vim_with_no_auth_url(self):
        cmd = vim.CreateVIM(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'test_vim'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config_without_auth_url.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        position_names = ['auth_cred', 'vim_project', 'auth_url', 'type']
        position_values = [self.auth_cred, self.vim_project,
                           self.auth_url, self.type]
        extra_body = {'type': 'openstack', 'name': name,
                      'description': description, 'is_default': False}
        message = 'Auth URL must be specified'
        ex = self.assertRaises(exceptions.TackerClientException,
                               self._test_create_resource,
                               self._RESOURCE, cmd, None, my_id, args,
                               position_names, position_values,
                               extra_body=extra_body)
        self.assertEqual(message, ex.message)
        self.assertEqual(404, ex.status_code)

    def test_register_vim_with_mandatory_params(self):
        cmd = vim.CreateVIM(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        my_id = 'my-id'

        vim_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config.yaml')
        args = [
            name,
            '--config-file', vim_config,
        ]
        position_names = ['auth_cred', 'vim_project', 'auth_url', 'type']
        position_values = [
            self.auth_cred,
            self.vim_project,
            self.auth_url,
            self.type
        ]
        extra_body = {'type': 'openstack', 'name': name, 'is_default': False}
        self._test_create_resource(self._RESOURCE, cmd, name, my_id, args,
                                   position_names, position_values,
                                   extra_body=extra_body)

    def test_list_vims(self):
        cmd = vim.ListVIM(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True)

    def test_show_vim_id(self):
        cmd = vim.ShowVIM(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def test_show_vim_id_name(self):
        cmd = vim.ShowVIM(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_vim_all_params(self):
        cmd = vim.UpdateVIM(test_cli10.MyApp(sys.stdout), None)
        update_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config_without_auth_url.yaml')
        my_id = 'my-id'
        name = 'new_name'
        description = 'new_description'
        is_default = 'True'
        args = [
            my_id,
            '--config-file', str(update_config),
            '--name', name,
            '--description', description,
            '--is_default', is_default]
        extra_fields = {'vim_project': self.vim_project,
                        'auth_cred': self.auth_cred,
                        'is_default': 'True',
                        'name': name, 'description': description}
        self._test_update_resource(self._RESOURCE, cmd, my_id, args,
                                   extra_fields)

    def test_update_vim_with_mandatory_params(self):
        cmd = vim.UpdateVIM(test_cli10.MyApp(sys.stdout), None)
        update_config = utils.get_file_path(
            'tests/unit/vm/samples/vim_config_without_auth_url.yaml')
        my_id = 'my-id'
        args = [
            my_id,
            '--config-file', str(update_config)]
        extra_fields = {'vim_project': self.vim_project,
                        'auth_cred': self.auth_cred}
        self._test_update_resource(self._RESOURCE, cmd, my_id, args,
                                   extra_fields)

    def test_delete_vim(self):
        cmd = vim.DeleteVIM(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)

    def test_multi_delete_vim(self):
        cmd = vim.DeleteVIM(test_cli10.MyApp(sys.stdout), None)
        vim_ids = 'my-id1 my-id2 my-id3'
        args = [vim_ids]
        self._test_delete_resource(self._RESOURCE, cmd, vim_ids, args)



# Copyright 2016 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tackerclient.common import exceptions
from tackerclient.tacker.v1_0.nfvo import vim_utils
from unittest.mock import sentinel


class TestVIMUtils(testtools.TestCase):

    def test_args2body_vim(self):
        config_param = {'project_name': sentinel.prj_name,
                        'username': sentinel.usrname1,
                        'password': sentinel.password1,
                        'project_domain_name': sentinel.prj_domain_name1,
                        'user_domain_name': sentinel.user_domain.name,
                        'cert_verify': 'True',
                        'type': 'openstack'}
        vim = {}
        auth_cred = config_param.copy()
        auth_cred.pop('project_name')
        auth_cred.pop('project_domain_name')
        auth_cred.pop('type')
        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name,
                             'project_domain_name': sentinel.prj_domain_name1},
                        'type': 'openstack'}
        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_vim_extra(self):
        auth_cred = {'username': sentinel.usrname1,
                     'password': sentinel.password1,
                     'user_domain_name': sentinel.user_domain.name,
                     'cert_verify': 'True'}
        config_param = {'project_name': sentinel.prj_name,
                        'project_domain_name': sentinel.prj_domain_name1,
                        'type': 'openstack',
                        'extra': {'area': 'area_A@region_A'},
                        **auth_cred}
        vim = {}

        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name,
                             'project_domain_name': sentinel.prj_domain_name1},
                        'type': 'openstack',
                        'extra': {'area': 'area_A@region_A'}}

        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_kubernetes_vim(self):
        config_param = {'username': sentinel.usrname1,
                        'password': sentinel.password1,
                        'ssl_ca_cert': 'abcxyz',
                        'project_name': sentinel.prj_name,
                        'type': 'kubernetes'}
        vim = {}
        auth_cred = config_param.copy()
        auth_cred.pop('project_name')
        auth_cred.pop('type')
        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name},
                        'type': 'kubernetes'}
        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_kubernetes_vim_bearer(self):
        config_param = {'bearer_token': sentinel.bearer_token,
                        'ssl_ca_cert': "None",
                        'project_name': sentinel.prj_name,
                        'type': 'kubernetes'}
        vim = {}
        auth_cred = config_param.copy()
        auth_cred.pop('project_name')
        auth_cred.pop('type')
        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name},
                        'type': 'kubernetes'}
        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_kubernetes_vim_oidc(self):
        config_param = {'oidc_token_url': sentinel.oidc_token_url,
                        'username': sentinel.username,
                        'password': sentinel.password,
                        'client_id': sentinel.client_id,
                        'client_secret': sentinel.client_secret,
                        'ssl_ca_cert': "None",
                        'project_name': sentinel.prj_name,
                        'type': 'kubernetes'}
        vim = {}
        auth_cred = config_param.copy()
        auth_cred.pop('project_name')
        auth_cred.pop('type')
        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name},
                        'type': 'kubernetes'}
        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_kubernetes_vim_extra(self):
        extra_param = {
            'helm_info': {
                'masternode_ip': [
                    '192.168.10.110'
                ],
                'masternode_username': 'helm_user',
                'masternode_password': 'helm_pass'
            }}

        config_param = {'username': sentinel.usrname1,
                        'password': sentinel.password1,
                        'ssl_ca_cert': 'abcxyz',
                        'project_name': sentinel.prj_name,
                        'type': 'kubernetes',
                        'extra': extra_param}
        vim = {}
        auth_cred = config_param.copy()
        auth_cred.pop('project_name')
        auth_cred.pop('type')
        auth_cred.pop('extra')
        expected_vim = {'auth_cred': auth_cred,
                        'vim_project':
                            {'name': sentinel.prj_name},
                        'type': 'kubernetes',
                        'extra': extra_param}
        vim_utils.args2body_vim(config_param.copy(), vim)
        self.assertEqual(expected_vim, vim)

    def test_args2body_kubernetes_vim_oidc_no_username(self):
        config_param = {'oidc_token_url': sentinel.oidc_token_url,
                        'password': sentinel.password,
                        'client_id': sentinel.client_id,
                        'client_secret': sentinel.client_secret,
                        'ssl_ca_cert': "None",
                        'project_name': sentinel.prj_name,
                        'type': 'kubernetes'}
        vim = {}
        self.assertRaises(exceptions.TackerClientException,
                          vim_utils.args2body_vim,
                          config_param, vim)

    def test_args2body_vim_no_project(self):
        config_param = {'username': sentinel.usrname1,
                        'password': sentinel.password1,
                        'user_domain_name': sentinel.user_domain.name,
                        'cert_verify': 'True',
                        'type': 'openstack'}
        vim = {}
        self.assertRaises(exceptions.TackerClientException,
                          vim_utils.args2body_vim,
                          config_param, vim)

    def test_validate_auth_url_with_port(self):
        auth_url = "http://localhost:8000/test"
        url_parts = vim_utils.validate_auth_url(auth_url)
        self.assertEqual('http', url_parts.scheme)
        self.assertEqual('localhost:8000', url_parts.netloc)
        self.assertEqual(8000, url_parts.port)

    def test_validate_auth_url_without_port(self):
        auth_url = "http://localhost/test"
        url_parts = vim_utils.validate_auth_url(auth_url)
        self.assertEqual('http', url_parts.scheme)
        self.assertEqual('localhost', url_parts.netloc)

    def test_validate_auth_url_exception(self):
        auth_url = "localhost/test"
        self.assertRaises(exceptions.TackerClientException,
                          vim_utils.validate_auth_url,
                          auth_url)



# Copyright 2016 Brocade Communications Systems Inc
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import yaml

from oslo_utils import strutils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10
from tackerclient.tacker.v1_0.nfvo import vim_utils

_VIM = "vim"


class ListVIM(tackerV10.ListCommand):
    """List VIMs that belong to a given tenant."""

    resource = _VIM
    list_columns = ['id', 'tenant_id', 'name', 'type', 'is_default',
                    'placement_attr', 'status']


class ShowVIM(tackerV10.ShowCommand):
    """Show information of a given VIM."""

    resource = _VIM


class CreateVIM(tackerV10.CreateCommand):
    """Create a VIM."""

    resource = _VIM

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--config-file',
            required=True,
            help=_('YAML file with VIM configuration parameters'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VIM'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the VIM'))
        parser.add_argument(
            '--is-default',
            action='store_true',
            default=False,
            help=_('Set as default VIM'))

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                vim_config = f.read()
                try:
                    config_param = yaml.load(vim_config,
                                             Loader=yaml.SafeLoader)
                except yaml.YAMLError as e:
                    raise exceptions.InvalidInput(reason=e)
        vim_obj = body[self.resource]
        try:
            auth_url = config_param.pop('auth_url')
        except KeyError:
            raise exceptions.TackerClientException(message='Auth URL must be '
                                                           'specified',
                                                   status_code=404)
        vim_obj['auth_url'] = vim_utils.validate_auth_url(auth_url).geturl()
        vim_utils.args2body_vim(config_param, vim_obj)
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description',
                               'is_default'])
        return body


class UpdateVIM(tackerV10.UpdateCommand):
    """Update a given VIM."""

    resource = _VIM

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--config-file',
            required=False,
            help=_('YAML file with VIM configuration parameters'))
        parser.add_argument(
            '--name',
            help=_('New name for the VIM'))
        parser.add_argument(
            '--description',
            help=_('New description for the VIM'))
        parser.add_argument(
            '--is-default',
            type=strutils.bool_from_string,
            metavar='{True,False}',
            help=_('Indicate whether the VIM is used as default'))

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        config_param = None
        # config arg passed as data overrides config yaml when both args passed
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                config_yaml = f.read()
            try:
                config_param = yaml.load(config_yaml,
                                         Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
        vim_obj = body[self.resource]
        if config_param is not None:
            vim_utils.args2body_vim(config_param, vim_obj)
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description',
                               'is_default'])
        # type attribute is read-only, it can't be updated, so remove it
        # in update method
        body['vim'].pop('type', None)
        return body


class DeleteVIM(tackerV10.DeleteCommand):
    """Delete given VIM(s)."""
    resource = _VIM



# Copyright (C) 2022 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils.fixture import uuidsentinel
from oslo_utils import uuidutils
from tackerclient.osc import utils as tacker_osc_utils


def lccn_subsc_response(attrs=None):
    """Create a fake subscription.

    :param Dictionary attrs:
        A dictionary with all attributes
    :return:
        A subscription dict
    """
    attrs = attrs or {}
    id = uuidsentinel.lccn_subsc_id

    # Set default attributes.
    dummy_subscription = {
        "id": id,
        "filter": {
            "vnfInstanceSubscriptionFilter": {
                "vnfdIds": [
                    "dummy-vnfdId-1",
                    "dummy-vnfdId-2"
                ],
                "vnfProductsFromProviders": [
                    {
                        "vnfProvider": "dummy-vnfProvider-1",
                        "vnfProducts": [
                            {
                                "vnfProductName": "dummy-vnfProductName-1-1",
                                "versions": [
                                    {
                                        "vnfSoftwareVersion": "1.0",
                                        "vnfdVersions": ["1.0", "2.0"]
                                    },
                                    {
                                        "vnfSoftwareVersion": "1.1",
                                        "vnfdVersions": ["1.1", "2.1"]
                                    }
                                ]
                            },
                            {
                                "vnfProductName": "dummy-vnfProductName-1-2",
                                "versions": [
                                    {
                                        "vnfSoftwareVersion": "1.0",
                                        "vnfdVersions": ["1.0", "2.0"]
                                    },
                                    {
                                        "vnfSoftwareVersion": "1.1",
                                        "vnfdVersions": ["1.1", "2.1"]
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        "vnfProvider": "dummy-vnfProvider-2",
                        "vnfProducts": [
                            {
                                "vnfProductName": "dummy-vnfProductName-2-1",
                                "versions": [
                                    {
                                        "vnfSoftwareVersion": "1.0",
                                        "vnfdVersions": ["1.0", "2.0"]
                                    },
                                    {
                                        "vnfSoftwareVersion": "1.1",
                                        "vnfdVersions": ["1.1", "2.1"]
                                    }
                                ]
                            },
                            {
                                "vnfProductName": "dummy-vnfProductName-2-2",
                                "versions": [
                                    {
                                        "vnfSoftwareVersion": "1.0",
                                        "vnfdVersions": ["1.0", "2.0"]
                                    },
                                    {
                                        "vnfSoftwareVersion": "1.1",
                                        "vnfdVersions": ["1.1", "2.1"]
                                    }
                                ]
                            }
                        ]
                    }
                ],
                "vnfInstanceIds": [
                    "dummy-vnfInstanceId-1",
                    "dummy-vnfInstanceId-2"
                ],
                "vnfInstanceNames": [
                    "dummy-vnfInstanceName-1",
                    "dummy-vnfInstanceName-2"
                ]
            },
            "notificationTypes": [
                "VnfLcmOperationOccurrenceNotification",
                "VnfIdentifierCreationNotification",
                "VnfIdentifierDeletionNotification"
            ],
            "operationTypes": [
                "INSTANTIATE",
                "SCALE",
                "TERMINATE",
                "HEAL",
                "MODIFY_INFO",
                "CHANGE_EXT_CONN"
            ],
            "operationStates": [
                "COMPLETED",
                "FAILED",
                "FAILED_TEMP",
                "PROCESSING",
                "ROLLING_BACK",
                "ROLLED_BACK",
                "STARTING"
            ]
        },
        "callbackUri": "http://localhost:9990/notification/callback/test",
        "_links": {
            "self": {
                "href": "http://127.0.0.1:9890/vnflcm/v2/subscriptions/" + id
            }
        }
    }

    # Overwrite default attributes.
    dummy_subscription.update(attrs)

    return dummy_subscription


def get_subscription_data(subscription, list_action=False, columns=None):
    """Get the subscription data.

    :return:
        A tuple object sorted based on the name of the columns.
    """
    subsc = subscription.copy()
    complex_attributes = ['filter', '_links']
    for attribute in complex_attributes:
        if subsc.get(attribute):
            subsc.update(
                {attribute: tacker_osc_utils.FormatComplexDataColumn(
                    subsc[attribute])})

    if list_action:
        for item in ['filter', '_links']:
            subsc.pop(item)

    # return the list of data as per column order
    if columns:
        return tuple([subsc[key] for key in columns])

    return tuple([subsc[key] for key in sorted(
        subsc.keys())])


def create_subscriptions(count=2):
    """Create multiple fake subscriptions.

    :param count: The number of subscriptions to fake
    :return:
        A list of fake subscriptions dictionary
    """
    uri = "http://localhost:9990/notification/callback/"
    subscriptions = []
    for i in range(0, count):
        unique_id = uuidutils.generate_uuid()
        subscriptions.append(lccn_subsc_response(
            attrs={'id': unique_id,
                   'callbackUri': uri + str(i)}))
    return subscriptions



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils.fixture import uuidsentinel
from oslo_utils import uuidutils

from tackerclient.osc import utils as tacker_osc_utils


def vnflcm_op_occ_response(attrs=None, action=''):
    """Create a fake vnflcm op occurrence.

    :param Dictionary attrs:
        A dictionary with all attributes
    :return:
        A vnf lcm op occs dict
    """
    attrs = attrs or {}

    # Set default attributes.
    dummy_vnf_lcm_op_occ = {
        "id": uuidsentinel.vnflcm_op_occ_id,
        "operationState": "STARTING",
        "stateEnteredTime": "2018-12-22T16:59:45.187Z",
        "startTime": "2018-12-22T16:59:45.187Z",
        "vnfInstanceId": "376f37f3-d4e9-4d41-8e6a-9b0ec98695cc",
        "grantId": "",
        "operation": "INSTANTIATE",
        "isAutomaticInvocation": "true",
        "operationParams": {
            "flavourId": "default",
            "instantiationLevelId": "n-mme-min"
        },
        "isCancelPending": "true",
        "cancelMode": "",
        "error": {
            "status": "500",
            "detail": "internal server error"
        },
        "resourceChanges": [],
        "changedInfo": [],
        "changedExtConnectivity": [],
        "_links": {
            "self": ""
        }
    }

    if action == 'fail':
        fail_not_needed_columns = [
            'grantId', 'operationParams',
            'cancelMode', 'resourceChanges', 'changedInfo',
            'changedExtConnectivity']

        for key in fail_not_needed_columns:
            del dummy_vnf_lcm_op_occ[key]

    # Overwrite default attributes.
    dummy_vnf_lcm_op_occ.update(attrs)

    return dummy_vnf_lcm_op_occ


def get_vnflcm_op_occ_data(vnf_lcm_op_occ, columns=None):
    """Get the vnflcm op occurrence.

    :return:
        A tuple object sorted based on the name of the columns.
    """
    complex_attributes = [
        'operationParams', 'error', 'resourceChanges',
        'changedInfo', 'changedExtConnectivity', 'links']

    for attribute in complex_attributes:
        if vnf_lcm_op_occ.get(attribute):
            vnf_lcm_op_occ.update(
                {attribute: tacker_osc_utils.FormatComplexDataColumn(
                    vnf_lcm_op_occ[attribute])})

    # return the list of data as per column order
    if columns:
        return tuple([vnf_lcm_op_occ[key] for key in columns])

    return tuple([vnf_lcm_op_occ[key] for key in sorted(
        vnf_lcm_op_occ.keys())])


def create_vnflcm_op_occs(count=2):
    """Create multiple fake vnflcm op occs.

    :param count: The number of vnflcm op occs to fake
    :return:
        A list of fake vnflcm op occs dictionary
    """
    vnflcm_op_occs = []
    for i in range(0, count):
        unique_id = uuidutils.generate_uuid()
        vnflcm_op_occs.append(vnflcm_op_occ_response(attrs={'id': unique_id}))
    return vnflcm_op_occs



# Copyright (C) 2022 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import os
import sys

from io import StringIO
from tackerclient.common import exceptions
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.osc.v1.vnflcm import vnflcm_subsc
from tackerclient.tests.unit.osc import base
from tackerclient.tests.unit.osc.v1 import test_vnflcm
from tackerclient.tests.unit.osc.v1 import vnflcm_subsc_fakes


def _get_columns_vnflcm_subsc(action=None):
    columns = ['ID', 'Filter', 'Callback URI', 'Links']
    if action == 'list':
        columns = [ele for ele in columns if ele not in
                   ['Filter', 'Links']]
    return columns


class TestCreateLccnSubscription(test_vnflcm.TestVnfLcm):

    def setUp(self):
        super(TestCreateLccnSubscription, self).setUp()
        self.create_subscription = vnflcm_subsc.CreateLccnSubscription(
            self.app, self.app_args, cmd_name='vnflcm subsc create')

    def test_create_no_args(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.create_subscription, [], [])

    def test_take_action(self):
        subscription = vnflcm_subsc_fakes.lccn_subsc_response()
        sample_param_file = ("./tackerclient/osc/v1/vnflcm/samples/"
                             "create_lccn_subscription_param_sample.json")

        arglist = [sample_param_file]
        verifylist = [('create_request_file', sample_param_file)]

        # command param
        parsed_args = self.check_parser(self.create_subscription,
                                        arglist, verifylist)

        self.requests_mock.register_uri(
            'POST', os.path.join(self.url, 'vnflcm/v1/subscriptions'),
            json=subscription, headers=self.header)

        actual_columns, data = (self.create_subscription.take_action(
                                parsed_args))

        headers, attributes = vnflcm_subsc._get_columns(subscription)

        self.assertCountEqual(_get_columns_vnflcm_subsc(),
                              actual_columns)
        self.assertListItemsEqual(vnflcm_subsc_fakes.get_subscription_data(
            subscription, columns=attributes), data)


class TestListLccnSubscription(test_vnflcm.TestVnfLcm):

    def setUp(self):
        super(TestListLccnSubscription, self).setUp()
        self.list_subscription = vnflcm_subsc.ListLccnSubscription(
            self.app, self.app_args, cmd_name='vnflcm subsc list')

    def test_take_action(self):
        subscriptions = vnflcm_subsc_fakes.create_subscriptions(count=3)
        parsed_args = self.check_parser(self.list_subscription, [], [])
        self.requests_mock.register_uri(
            'GET', os.path.join(self.url, 'vnflcm/v1/subscriptions'),
            json=subscriptions, headers=self.header)
        actual_columns, data = self.list_subscription.take_action(parsed_args)

        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_subscription.get_attributes(), long_listing=True)

        expected_data = []
        for subscription_obj in subscriptions:
            expected_data.append(vnflcm_subsc_fakes.get_subscription_data(
                subscription_obj, columns=columns, list_action=True))

        self.assertCountEqual(_get_columns_vnflcm_subsc(action='list'),
                              actual_columns)
        self.assertCountEqual(expected_data, list(data))

    def test_take_action_with_pagination(self):
        subscriptions = vnflcm_subsc_fakes.create_subscriptions(count=3)
        next_links_num = 3
        path = os.path.join(self.url, 'vnflcm/v1/subscriptions')
        parsed_args = self.check_parser(self.list_subscription, [], [])

        links = [0] * next_links_num
        link_headers = [0] * next_links_num

        for i in range(next_links_num):
            links[i] = (
                '{base_url}?nextpage_opaque_marker={subscription_id}'.format(
                    base_url=path,
                    subscription_id=subscriptions[i]['id']))

            link_headers[i] = copy.deepcopy(self.header)
            link_headers[i]['Link'] = '<{link_url}>; rel="next"'.format(
                link_url=links[i])

        self.requests_mock.register_uri(
            'GET', path, json=[subscriptions[0]], headers=link_headers[0])
        self.requests_mock.register_uri(
            'GET', links[0], json=[subscriptions[1]],
            headers=link_headers[1])
        self.requests_mock.register_uri(
            'GET', links[1], json=[subscriptions[2]],
            headers=link_headers[2])
        self.requests_mock.register_uri(
            'GET', links[2], json=[], headers=self.header)

        actual_columns, data = self.list_subscription.take_action(parsed_args)

        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_subscription.get_attributes(), long_listing=True)

        expected_data = []
        for subscription_obj in subscriptions:
            expected_data.append(vnflcm_subsc_fakes.get_subscription_data(
                subscription_obj, columns=columns, list_action=True))

        self.assertCountEqual(_get_columns_vnflcm_subsc(action='list'),
                              actual_columns)

        self.assertCountEqual(expected_data, list(data))


class TestShowLccnSubscription(test_vnflcm.TestVnfLcm):

    def setUp(self):
        super(TestShowLccnSubscription, self).setUp()
        self.show_subscription = vnflcm_subsc.ShowLccnSubscription(
            self.app, self.app_args, cmd_name='vnflcm subsc show')

    def test_take_action(self):
        subscription = vnflcm_subsc_fakes.lccn_subsc_response()

        arglist = [subscription['id']]
        verifylist = [('subscription_id', subscription['id'])]

        # command param
        parsed_args = self.check_parser(self.show_subscription, arglist,
                                        verifylist)

        self.requests_mock.register_uri(
            'GET', os.path.join(self.url, 'vnflcm/v1/subscriptions',
                                subscription['id']),
            json=subscription, headers=self.header)

        columns, data = (self.show_subscription.take_action(parsed_args))
        self.assertCountEqual(_get_columns_vnflcm_subsc(),
                              columns)
        headers, attributes = vnflcm_subsc._get_columns(subscription)
        self.assertListItemsEqual(
            vnflcm_subsc_fakes.get_subscription_data(subscription,
                                                     columns=attributes),
            data)


class TestDeleteLccnSubscription(test_vnflcm.TestVnfLcm):

    subscriptions = vnflcm_subsc_fakes.create_subscriptions(count=3)

    def setUp(self):
        super(TestDeleteLccnSubscription, self).setUp()
        self.delete_subscription = vnflcm_subsc.DeleteLccnSubscription(
            self.app, self.app_args, cmd_name='vnflcm subsc delete')

    def _mock_request_url_for_delete(self, subsc_index):
        url = os.path.join(self.url, 'vnflcm/v1/subscriptions',
                           self.subscriptions[subsc_index]['id'])

        json = self.subscriptions[subsc_index]

        self.requests_mock.register_uri('GET', url, json=json,
                                        headers=self.header)
        self.requests_mock.register_uri('DELETE', url,
                                        headers=self.header, json={})

    def test_delete_one_subscription(self):
        arglist = [self.subscriptions[0]['id']]
        verifylist = [('subscription_id',
                       [self.subscriptions[0]['id']])]

        parsed_args = self.check_parser(self.delete_subscription,
                                        arglist, verifylist)

        self._mock_request_url_for_delete(0)
        sys.stdout = buffer = StringIO()
        result = self.delete_subscription.take_action(parsed_args)
        self.assertIsNone(result)
        self.assertEqual(("Lccn Subscription '%s' is deleted successfully")
                         % self.subscriptions[0]['id'],
                         buffer.getvalue().strip())

    def test_delete_multiple_subscription(self):
        arglist = []
        for subscription in self.subscriptions:
            arglist.append(subscription['id'])
        verifylist = [('subscription_id', arglist)]
        parsed_args = self.check_parser(self.delete_subscription,
                                        arglist, verifylist)
        for i in range(0, 3):
            self._mock_request_url_for_delete(i)
        sys.stdout = buffer = StringIO()
        result = self.delete_subscription.take_action(parsed_args)
        self.assertIsNone(result)
        self.assertEqual('All specified Lccn Subscriptions are deleted '
                         'successfully', buffer.getvalue().strip())

    def test_delete_multiple_subscription_exception(self):
        arglist = [
            self.subscriptions[0]['id'],
            'xxxx-yyyy-zzzz',
            self.subscriptions[1]['id'],
        ]
        verifylist = [('subscription_id', arglist)]
        parsed_args = self.check_parser(self.delete_subscription,
                                        arglist, verifylist)

        self._mock_request_url_for_delete(0)

        url = os.path.join(self.url, 'vnflcm/v1/subscriptions',
                           'xxxx-yyyy-zzzz')
        self.requests_mock.register_uri(
            'GET', url, exc=exceptions.ConnectionFailed)

        self._mock_request_url_for_delete(1)
        exception = self.assertRaises(exceptions.CommandError,
                                      self.delete_subscription.take_action,
                                      parsed_args)

        self.assertEqual('Failed to delete 1 of 3 Lccn Subscriptions.',
                         exception.message)



# Copyright 2016 Brocade Communications Systems Inc
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urlparse

from tackerclient.common import exceptions


def args2body_vim(config_param, vim):
    """Create additional args to vim body

    :param vim: vim request object
    :return: vim body with args populated
    """
    vim_type = ['openstack', 'kubernetes']
    cert_verify_type = ['True', 'False']

    if 'type' in config_param:
        vim['type'] = config_param.pop('type', '')
        if not vim['type'] in vim_type:
            raise exceptions.TackerClientException(
                message='Supported VIM types: openstack, kubernetes',
                status_code=400)
    else:
        vim['type'] = 'openstack'
    if vim['type'] == 'openstack':
        vim['vim_project'] = {
            'name': config_param.pop('project_name', ''),
            'project_domain_name':
                config_param.pop('project_domain_name', '')}
        if not vim['vim_project']['name']:
            raise exceptions.TackerClientException(
                message='Project name must be specified',
                status_code=404)
        cert_verify = config_param.pop('cert_verify', 'True')
        if cert_verify not in cert_verify_type:
            raise exceptions.TackerClientException(
                message='Supported cert_verify types: True, False',
                status_code=400)
        vim['auth_cred'] = {'username': config_param.pop('username', ''),
                            'password': config_param.pop('password', ''),
                            'user_domain_name':
                                config_param.pop('user_domain_name', ''),
                            'cert_verify': cert_verify}
    elif vim['type'] == 'kubernetes':
        vim['vim_project'] = {
            'name': config_param.pop('project_name', '')}
        if not vim['vim_project']['name']:
            raise exceptions.TackerClientException(
                message='Project name must be specified in Kubernetes VIM,'
                        'it is namespace in Kubernetes environment',
                status_code=404)
        if 'oidc_token_url' in config_param:
            if ('username' not in config_param or
                    'password' not in config_param or
                    'client_id' not in config_param):
                # the username, password, client_id are required.
                # client_secret is not required when client type is public.
                raise exceptions.TackerClientException(
                    message='oidc_token_url must be specified with username,'
                            ' password, client_id, client_secret(optional).',
                    status_code=404)
            vim['auth_cred'] = {
                'oidc_token_url': config_param.pop('oidc_token_url'),
                'username': config_param.pop('username'),
                'password': config_param.pop('password'),
                'client_id': config_param.pop('client_id')}
            if 'client_secret' in config_param:
                vim['auth_cred']['client_secret'] = config_param.pop(
                    'client_secret')
        elif ('username' in config_param) and ('password' in config_param):
            vim['auth_cred'] = {
                'username': config_param.pop('username', ''),
                'password': config_param.pop('password', '')}
        elif 'bearer_token' in config_param:
            vim['auth_cred'] = {
                'bearer_token': config_param.pop('bearer_token', '')}
        else:
            raise exceptions.TackerClientException(
                message='username and password or bearer_token must be'
                        'provided',
                status_code=404)
        ssl_ca_cert = config_param.pop('ssl_ca_cert', '')
        if ssl_ca_cert:
            vim['auth_cred']['ssl_ca_cert'] = ssl_ca_cert
    if 'extra' in config_param:
        vim['extra'] = config_param.pop('extra')


def validate_auth_url(url):
    url_parts = urlparse.urlparse(url)
    if not url_parts.scheme or not url_parts.netloc:
        raise exceptions.TackerClientException(message='Invalid auth URL')
    return url_parts



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osc_lib.command import command

from tackerclient.common import exceptions
from tackerclient.i18n import _


SUPPORTED_VERSIONS = [1, 2]


class VnfLcmVersions(command.ShowOne):
    _description = _("Show VnfLcm Api versions")

    def get_parser(self, prog_name):
        parser = super(VnfLcmVersions, self).get_parser(prog_name)
        parser.add_argument(
            '--major-version',
            metavar="<major-version>",
            type=int,
            help=_('Show only specify major version.'))
        return parser

    def take_action(self, parsed_args):
        v = None
        if parsed_args.major_version:
            if parsed_args.major_version not in SUPPORTED_VERSIONS:
                msg = _("Major version %d is not supported")
                reason = msg % parsed_args.major_version
                raise exceptions.InvalidInput(reason=reason)
            v = "v{}".format(parsed_args.major_version)

        client = self.app.client_manager.tackerclient
        data = client.show_vnf_lcm_versions(v)

        return (tuple(data.keys()), tuple(data.values()))
